@@ -5,22 +5,24 @@
  */
 
 var Fetch = {
-  // Map of integer fetch id to XHR request object
-  xhrs: {},
+  // HandleAllocator for XHR request object
+  // xhrs: undefined,
 
-  // The web worker that runs proxied file I/O requests. (this field is populated on demand, start as undefined to save code size)
+  // The web worker that runs proxied file I/O requests. (this field is
+  // populated on demand, start as undefined to save code size)
   // worker: undefined,
 
   // Specifies an instance to the IndexedDB database. The database is opened
-  // as a preload step before the Emscripten application starts. (this field is populated on demand, start as undefined to save code size)
+  // as a preload step before the Emscripten application starts. (this field is
+  // populated on demand, start as undefined to save code size)
   // dbInstance: undefined,
 
 #if FETCH_SUPPORT_INDEXEDDB
   // Be cautious that `onerror` may be run synchronously
-  openDatabase: function(dbname, dbversion, onsuccess, onerror) {
+  openDatabase(dbname, dbversion, onsuccess, onerror) {
     try {
 #if FETCH_DEBUG
-      dbg('fetch: indexedDB.open(dbname="' + dbname + '", dbversion="' + dbversion + '");');
+      dbg(`fetch: indexedDB.open(dbname="${dbname}", dbversion="${dbversion}");`);
 #endif
       var openRequest = indexedDB.open(dbname, dbversion);
     } catch (e) { return onerror(e); }
@@ -40,8 +42,12 @@ var Fetch = {
   },
 #endif
 
-  staticInit: function() {
+  init() {
+    Fetch.xhrs = new HandleAllocator();
 #if FETCH_SUPPORT_INDEXEDDB
+#if PTHREADS
+    if (ENVIRONMENT_IS_PTHREAD) return;
+#endif
     var onsuccess = (db) => {
 #if FETCH_DEBUG
       dbg('fetch: IndexedDB successfully opened.');
@@ -86,7 +92,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     request.onsuccess = (event) => {
       var value = event.target.result;
 #if FETCH_DEBUG
-      dbg('fetch: Deleted file ' + pathStr + ' from IndexedDB');
+      dbg(`fetch: Deleted file ${pathStr} from IndexedDB`);
 #endif
       HEAPU32[fetch + {{{ C_STRUCTS.emscripten_fetch_t.data }}} >> 2] = 0;
       writeI53ToI64(fetch + {{{ C_STRUCTS.emscripten_fetch_t.numBytes }}}, 0);
@@ -99,7 +105,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     };
     request.onerror = (error) => {
 #if FETCH_DEBUG
-      dbg('fetch: Failed to delete file ' + pathStr + ' from IndexedDB! error: ' + error);
+      dbg(`fetch: Failed to delete file ${pathStr} from IndexedDB! error: ${error}`);
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -108,7 +114,7 @@ function fetchDeleteCachedData(db, fetch, onsuccess, onerror) {
     };
   } catch(e) {
 #if FETCH_DEBUG
-    dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
+    dbg(`fetch: Failed to load file ${pathStr} from IndexedDB! Got exception ${e}`);
 #endif
     onerror(fetch, 0, e);
   }
@@ -137,7 +143,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
         var value = event.target.result;
         var len = value.byteLength || value.length;
 #if FETCH_DEBUG
-        dbg('fetch: Loaded file ' + pathStr + ' from IndexedDB, length: ' + len);
+        dbg(`fetch: Loaded file ${pathStr} from IndexedDB, length: ${len}`);
 #endif
         // The data pointer malloc()ed here has the same lifetime as the emscripten_fetch_t structure itself has, and is
         // freed when emscripten_fetch_close() is called.
@@ -154,7 +160,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
       } else {
         // Succeeded to load, but the load came back with the value of undefined, treat that as an error since we never store undefined in db.
 #if FETCH_DEBUG
-        dbg('fetch: File ' + pathStr + ' not found in IndexedDB');
+        dbg(`fetch: File ${pathStr} not found in IndexedDB`);
 #endif
         HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
         HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -164,7 +170,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
     };
     getRequest.onerror = (error) => {
 #if FETCH_DEBUG
-      dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB!');
+      dbg(`fetch: Failed to load file ${pathStr} from IndexedDB!`);
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 404; // Mimic XHR HTTP status code 404 "Not Found"
@@ -173,7 +179,7 @@ function fetchLoadCachedData(db, fetch, onsuccess, onerror) {
     };
   } catch(e) {
 #if FETCH_DEBUG
-    dbg('fetch: Failed to load file ' + pathStr + ' from IndexedDB! Got exception ' + e);
+    dbg(`fetch: Failed to load file ${pathStr} from IndexedDB! Got exception ${e}`);
 #endif
     onerror(fetch, 0, e);
   }
@@ -199,7 +205,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     var putRequest = packages.put(data, destinationPathStr);
     putRequest.onsuccess = (event) => {
 #if FETCH_DEBUG
-      dbg('fetch: Stored file "' + destinationPathStr + '" to IndexedDB cache.');
+      dbg(`fetch: Stored file "${destinationPathStr}" to IndexedDB cache.`);
 #endif
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.readyState }}} >> 1] = 4; // Mimic XHR readyState 4 === 'DONE: The operation is complete'
       HEAPU16[fetch + {{{ C_STRUCTS.emscripten_fetch_t.status }}} >> 1] = 200; // Mimic XHR HTTP status code 200 "OK"
@@ -208,7 +214,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     };
     putRequest.onerror = (error) => {
 #if FETCH_DEBUG
-      dbg('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache!');
+      dbg(`fetch: Failed to store file "${destinationPathStr}" to IndexedDB cache!`);
 #endif
       // Most likely we got an error if IndexedDB is unwilling to store any more data for this page.
       // TODO: Can we identify and break down different IndexedDB-provided errors and convert those
@@ -220,7 +226,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
     };
   } catch(e) {
 #if FETCH_DEBUG
-      dbg('fetch: Failed to store file "' + destinationPathStr + '" to IndexedDB cache! Exception: ' + e);
+      dbg(`fetch: Failed to store file "${destinationPathStr}" to IndexedDB cache! Exception: ${e}`);
 #endif
     onerror(fetch, 0, e);
   }
@@ -228,7 +234,7 @@ function fetchCacheData(/** @type {IDBDatabase} */ db, fetch, data, onsuccess, o
 #endif // ~FETCH_SUPPORT_INDEXEDDB
 
 function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
-  var url = HEAPU32[fetch + {{{ C_STRUCTS.emscripten_fetch_t.url }}} >> 2];
+  var url = {{{ makeGetValue('fetch', C_STRUCTS.emscripten_fetch_t.url, '*') }}};
   if (!url) {
 #if FETCH_DEBUG
     dbg('fetch: XHR failed, no URL specified!');
@@ -239,13 +245,9 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   var url_ = UTF8ToString(url);
 
   var fetch_attr = fetch + {{{ C_STRUCTS.emscripten_fetch_t.__attributes }}};
-  var requestMethod = UTF8ToString(fetch_attr);
+  var requestMethod = UTF8ToString(fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.requestMethod }}});
   if (!requestMethod) requestMethod = 'GET';
-  var userData = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_t.userData, '*') }}};
-  var fetchAttributes = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.attributes, 'u32') }}};
   var timeoutMsecs = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.timeoutMSecs, 'u32') }}};
-  var withCredentials = !!{{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.withCredentials, 'u8') }}};
-  var destinationPath = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.destinationPath, '*') }}};
   var userName = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.userName, '*') }}};
   var password = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.password, '*') }}};
   var requestHeaders = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.requestHeaders, '*') }}};
@@ -253,24 +255,19 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   var dataPtr = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.requestData, '*') }}};
   var dataLength = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.requestDataSize, '*') }}};
 
+  var fetchAttributes = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.attributes, 'u32') }}};
   var fetchAttrLoadToMemory = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_LOAD_TO_MEMORY }}});
   var fetchAttrStreamData = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_STREAM_DATA }}});
-#if FETCH_SUPPORT_INDEXEDDB
-  var fetchAttrPersistFile = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_PERSIST_FILE }}});
-#endif
-  var fetchAttrAppend = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_APPEND }}});
-  var fetchAttrReplace = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_REPLACE }}});
   var fetchAttrSynchronous = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_SYNCHRONOUS }}});
-  var fetchAttrWaitable = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_WAITABLE }}});
 
   var userNameStr = userName ? UTF8ToString(userName) : undefined;
   var passwordStr = password ? UTF8ToString(password) : undefined;
 
   var xhr = new XMLHttpRequest();
-  xhr.withCredentials = withCredentials;
+  xhr.withCredentials = !!{{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.withCredentials, 'u8') }}};;
 #if FETCH_DEBUG
-  dbg('fetch: xhr.timeout: ' + xhr.timeout + ', xhr.withCredentials: ' + xhr.withCredentials);
-  dbg('fetch: xhr.open(requestMethod="' + requestMethod + '", url: "' + url_ +'", userName: ' + userNameStr + ', password: ' + passwordStr + ');');
+  dbg(`fetch: xhr.timeout: ${xhr.timeout}, xhr.withCredentials: ${xhr.withCredentials}`);
+  dbg(`fetch: xhr.open(requestMethod="${requestMethod}", url: "${url}", userName: ${userNameStr}, password: ${passwordStr}`);
 #endif
   xhr.open(requestMethod, url_, !fetchAttrSynchronous, userNameStr, passwordStr);
   if (!fetchAttrSynchronous) xhr.timeout = timeoutMsecs; // XHR timeout field is only accessible in async XHRs, and must be set after .open() but before .send().
@@ -283,7 +280,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   if (overriddenMimeType) {
     var overriddenMimeTypeStr = UTF8ToString(overriddenMimeType);
 #if FETCH_DEBUG
-    dbg('fetch: xhr.overrideMimeType("' + overriddenMimeTypeStr + '");');
+    dbg(`fetch: xhr.overrideMimeType("${overriddenMimeTypeStr}");`);
 #endif
     xhr.overrideMimeType(overriddenMimeTypeStr);
   }
@@ -297,13 +294,17 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
       var keyStr = UTF8ToString(key);
       var valueStr = UTF8ToString(value);
 #if FETCH_DEBUG
-      dbg('fetch: xhr.setRequestHeader("' + keyStr + '", "' + valueStr + '");');
+      dbg(`fetch: xhr.setRequestHeader("${keyStr}", "${valueStr}");`);
 #endif
       xhr.setRequestHeader(keyStr, valueStr);
     }
   }
-  var id = {{{ makeGetValue('fetch', C_STRUCTS.emscripten_fetch_t.id, 'u32') }}};
-  Fetch.xhrs[id] = xhr;
+
+  var id = Fetch.xhrs.allocate(xhr);
+#if FETCH_DEBUG
+  dbg(`fetch: id=${id}`);
+#endif
+  {{{ makeSetValue('fetch', C_STRUCTS.emscripten_fetch_t.id, 'id', 'u32') }}};
   var data = (dataPtr && dataLength) ? HEAPU8.slice(dataPtr, dataPtr + dataLength) : null;
   // TODO: Support specifying custom headers to the request.
 
@@ -319,7 +320,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     }
     if (ptrLen > 0) {
 #if FETCH_DEBUG
-      dbg('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
+      dbg(`fetch: allocating ${ptrLen} bytes in Emscripten heap for xhr data`);
 #endif
       // The data pointer malloc()ed here has the same lifetime as the emscripten_fetch_t structure itself has, and is
       // freed when emscripten_fetch_close() is called.
@@ -343,53 +344,53 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
 
   xhr.onload = (e) => {
     // check if xhr was aborted by user and don't try to call back
-    if (!(id in Fetch.xhrs)) { 
+    if (!Fetch.xhrs.has(id)) {
       return;
     }
     saveResponseAndStatus();
     if (xhr.status >= 200 && xhr.status < 300) {
 #if FETCH_DEBUG
-      dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" succeeded with status ' + xhr.status);
+      dbg(`fetch: xhr of URL "${xhr.url_}" / responseURL "${xhr.responseURL}" succeeded with status ${xhr.status}`);
 #endif
       if (onsuccess) onsuccess(fetch, xhr, e);
     } else {
 #if FETCH_DEBUG
-      dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" failed with status ' + xhr.status);
+      dbg(`fetch: xhr of URL "${xhr.url_}" / responseURL "${xhr.responseURL}" failed with status ${xhr.status}`);
 #endif
       if (onerror) onerror(fetch, xhr, e);
     }
   };
   xhr.onerror = (e) => {
     // check if xhr was aborted by user and don't try to call back
-    if (!(id in Fetch.xhrs)) { 
+    if (!Fetch.xhrs.has(id)) {
       return;
     }
 #if FETCH_DEBUG
-    dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" finished with error, readyState ' + xhr.readyState + ' and status ' + xhr.status);
+    dbg(`fetch: xhr of URL "${xhr.url_}" / responseURL "${xhr.responseURL}" finished with error, readyState ${xhr.readyState} and status ${xhr.status}`);
 #endif
     saveResponseAndStatus();
     if (onerror) onerror(fetch, xhr, e);
   };
   xhr.ontimeout = (e) => {
     // check if xhr was aborted by user and don't try to call back
-    if (!(id in Fetch.xhrs)) { 
+    if (!Fetch.xhrs.has(id)) {
       return;
     }
 #if FETCH_DEBUG
-    dbg('fetch: xhr of URL "' + xhr.url_ + '" / responseURL "' + xhr.responseURL + '" timed out, readyState ' + xhr.readyState + ' and status ' + xhr.status);
+    dbg(`fetch: xhr of URL "${xhr.url_}" / responseURL "${xhr.responseURL}" timed out, readyState ${xhr.readyState} and status ${xhr.status}`);
 #endif
     if (onerror) onerror(fetch, xhr, e);
   };
   xhr.onprogress = (e) => {
     // check if xhr was aborted by user and don't try to call back
-    if (!(id in Fetch.xhrs)) { 
+    if (!Fetch.xhrs.has(id)) {
       return;
     }
     var ptrLen = (fetchAttrLoadToMemory && fetchAttrStreamData && xhr.response) ? xhr.response.byteLength : 0;
     var ptr = 0;
     if (ptrLen > 0 && fetchAttrLoadToMemory && fetchAttrStreamData) {
 #if FETCH_DEBUG
-      dbg('fetch: allocating ' + ptrLen + ' bytes in Emscripten heap for xhr data');
+      dbg(`fetch: allocating ${ptrLen} bytes in Emscripten heap for xhr data`);
 #endif
 #if ASSERTIONS
       assert(onprogress, 'When doing a streaming fetch, you should have an onprogress handler registered to receive the chunks!');
@@ -414,7 +415,7 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
   };
   xhr.onreadystatechange = (e) => {
     // check if xhr was aborted by user and don't try to call back
-    if (!(id in Fetch.xhrs)) {
+    if (!Fetch.xhrs.has(id)) {
       {{{ runtimeKeepalivePop() }}}
       return;
     }
@@ -425,13 +426,13 @@ function fetchXHR(fetch, onsuccess, onerror, onprogress, onreadystatechange) {
     if (onreadystatechange) onreadystatechange(fetch, xhr, e);
   };
 #if FETCH_DEBUG
-  dbg('fetch: xhr.send(data=' + data + ')');
+  dbg(`fetch: xhr.send(data=${data})`);
 #endif
   try {
     xhr.send(data);
   } catch(e) {
 #if FETCH_DEBUG
-    dbg('fetch: xhr failed with exception: ' + e);
+    dbg(`fetch: xhr failed with exception: ${e}`);
 #endif
     if (onerror) onerror(fetch, xhr, e);
   }
@@ -443,20 +444,11 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
   {{{ runtimeKeepalivePush() }}}
 
   var fetch_attr = fetch + {{{ C_STRUCTS.emscripten_fetch_t.__attributes }}};
-  var requestMethod = UTF8ToString(fetch_attr);
   var onsuccess = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.onsuccess }}} >> 2];
   var onerror = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.onerror }}} >> 2];
   var onprogress = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.onprogress }}} >> 2];
   var onreadystatechange = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.onreadystatechange }}} >> 2];
   var fetchAttributes = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.attributes }}} >> 2];
-  var fetchAttrLoadToMemory = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_LOAD_TO_MEMORY }}});
-  var fetchAttrStreamData = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_STREAM_DATA }}});
-#if FETCH_SUPPORT_INDEXEDDB
-  var fetchAttrPersistFile = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_PERSIST_FILE }}});
-  var fetchAttrNoDownload = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_NO_DOWNLOAD }}});
-#endif
-  var fetchAttrAppend = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_APPEND }}});
-  var fetchAttrReplace = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_REPLACE }}});
   var fetchAttrSynchronous = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_SYNCHRONOUS }}});
 
   function doCallback(f) {
@@ -469,7 +461,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: operation success. e: ' + e);
+    dbg(`fetch: operation success. e: ${e}`);
 #endif
     {{{ runtimeKeepalivePop() }}}
     doCallback(() => {
@@ -487,7 +479,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportError = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: operation failed: ' + e);
+    dbg(`fetch: operation failed: ${e}`);
 #endif
     {{{ runtimeKeepalivePop() }}}
     doCallback(() => {
@@ -498,7 +490,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var reportReadyStateChange = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: ready state change. e: ' + e);
+    dbg(`fetch: ready state change. e: ${e}`);
 #endif
     doCallback(() => {
       if (onreadystatechange) {{{ makeDynCall('vp', 'onreadystatechange') }}}(fetch);
@@ -508,7 +500,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var performUncachedXhr = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: starting (uncached) XHR: ' + e);
+    dbg(`fetch: starting (uncached) XHR: ${e}`);
 #endif
     fetchXHR(fetch, reportSuccess, reportError, reportProgress, reportReadyStateChange);
   };
@@ -516,7 +508,7 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 #if FETCH_SUPPORT_INDEXEDDB
   var cacheResultAndReportSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: operation success. Caching result.. e: ' + e);
+    dbg(`fetch: operation success. Caching result.. e: ${e}`);
 #endif
     var storeSuccess = (fetch, xhr, e) => {
 #if FETCH_DEBUG
@@ -543,15 +535,20 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 
   var performCachedXhr = (fetch, xhr, e) => {
 #if FETCH_DEBUG
-    dbg('fetch: starting (cached) XHR: ' + e);
+    dbg(`fetch: starting (cached) XHR: ${e}`);
 #endif
     fetchXHR(fetch, cacheResultAndReportSuccess, reportError, reportProgress, reportReadyStateChange);
   };
 
+  var requestMethod = UTF8ToString(fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.requestMethod }}});
+  var fetchAttrReplace = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_REPLACE }}});
+  var fetchAttrPersistFile = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_PERSIST_FILE }}});
+  var fetchAttrNoDownload = !!(fetchAttributes & {{{ cDefs.EMSCRIPTEN_FETCH_NO_DOWNLOAD }}});
   if (requestMethod === 'EM_IDB_STORE') {
     // TODO(?): Here we perform a clone of the data, because storing shared typed arrays to IndexedDB does not seem to be allowed.
-    var ptr = HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.requestData }}} >> 2];
-    fetchCacheData(Fetch.dbInstance, fetch, HEAPU8.slice(ptr, ptr + HEAPU32[fetch_attr + {{{ C_STRUCTS.emscripten_fetch_attr_t.requestDataSize }}} >> 2]), reportSuccess, reportError);
+    var ptr = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.requestData, '*') }}};
+    var size = {{{ makeGetValue('fetch_attr', C_STRUCTS.emscripten_fetch_attr_t.requestDataSize, '*') }}};
+    fetchCacheData(Fetch.dbInstance, fetch, HEAPU8.slice(ptr, ptr + size), reportSuccess, reportError);
   } else if (requestMethod === 'EM_IDB_DELETE') {
     fetchDeleteCachedData(Fetch.dbInstance, fetch, reportSuccess, reportError);
   } else if (!fetchAttrReplace) {
@@ -572,27 +569,27 @@ function startFetch(fetch, successcb, errorcb, progresscb, readystatechangecb) {
 }
 
 function fetchGetResponseHeadersLength(id) {
-    return lengthBytesUTF8(Fetch.xhrs[id].getAllResponseHeaders()) + 1;
+  return lengthBytesUTF8(Fetch.xhrs.get(id).getAllResponseHeaders()) + 1;
 }
 
 function fetchGetResponseHeaders(id, dst, dstSizeBytes) {
-    var responseHeaders = Fetch.xhrs[id].getAllResponseHeaders();
-    var lengthBytes = lengthBytesUTF8(responseHeaders) + 1;
-    stringToUTF8(responseHeaders, dst, dstSizeBytes);
-    return Math.min(lengthBytes, dstSizeBytes);
+  var responseHeaders = Fetch.xhrs.get(id).getAllResponseHeaders();
+  var lengthBytes = lengthBytesUTF8(responseHeaders) + 1;
+  stringToUTF8(responseHeaders, dst, dstSizeBytes);
+  return Math.min(lengthBytes, dstSizeBytes);
 }
 
 //Delete the xhr JS object, allowing it to be garbage collected.
 function fetchFree(id) {
 #if FETCH_DEBUG
-    dbg("fetch: Deleting id:" + id + " of " + Fetch.xhrs);
+  dbg(`fetch: fetchFree id:${id}`);
 #endif
-    var xhr = Fetch.xhrs[id];
-    if (xhr) {
-        delete Fetch.xhrs[id];
-        // check if fetch is still in progress and should be aborted
-        if (xhr.readyState > 0 && xhr.readyState < 4) {
-            xhr.abort();
-        }
+  if (Fetch.xhrs.has(id)) {
+    var xhr = Fetch.xhrs.get(id);
+    Fetch.xhrs.free(id);
+    // check if fetch is still in progress and should be aborted
+    if (xhr.readyState > 0 && xhr.readyState < 4) {
+      xhr.abort();
     }
+  }
 }
