@@ -5,63 +5,55 @@
 
 /*global Module:true, Runtime*/
 /*global HEAP32*/
-/*global new_*/
+/*global newFunc*/
 /*global createNamedFunction*/
 /*global readLatin1String, stringToUTF8*/
 /*global requireRegisteredType, throwBindingError, runDestructors*/
 /*jslint sub:true*/ /* The symbols 'fromWireType' and 'toWireType' must be accessed via array notation to be closure-safe since craftInvokerFunction crafts functions as strings that can't be closured. */
 
 // -- jshint doesn't understand library syntax, so we need to mark the symbols exposed here
-/*global getStringOrSymbol, emval_handle_array, Emval, __emval_unregister, count_emval_handles, emval_symbols, emval_free_list, get_first_emval, __emval_decref, emval_newers*/
-/*global craftEmvalAllocator, emval_addMethodCaller, emval_methodCallers, LibraryManager, mergeInto, emval_allocateDestructors, global, emval_lookupTypes, makeLegalFunctionName*/
+/*global getStringOrSymbol, emval_handles, Emval, __emval_unregister, count_emval_handles, emval_symbols, __emval_decref, emval_newers*/
+/*global craftEmvalAllocator, emval_addMethodCaller, emval_methodCallers, addToLibrary, emval_allocateDestructors, global, emval_lookupTypes, makeLegalFunctionName*/
 /*global emval_get_global*/
 
 var LibraryEmVal = {
-  $emval_handle_array: [
-    {},
-    {value: undefined},
-    {value: null},
-    {value: true},
-    {value: false}
-  ], // reserve zero and special values
-  $emval_free_list: [],
+  $emval_handles__deps: ['$HandleAllocator'],
+  $emval_handles: "new HandleAllocator();",
   $emval_symbols: {}, // address -> string
 
-  $init_emval__deps: ['$count_emval_handles', '$get_first_emval'],
+  $init_emval__deps: ['$count_emval_handles', '$emval_handles'],
   $init_emval__postset: 'init_emval();',
-  $init_emval: function() {
+  $init_emval: () => {
+    // reserve some special values. These never get de-allocated.
+    // The HandleAllocator takes care of reserving zero.
+    emval_handles.allocated.push(
+      {value: undefined},
+      {value: null},
+      {value: true},
+      {value: false},
+    );
+    emval_handles.reserved = emval_handles.allocated.length
     Module['count_emval_handles'] = count_emval_handles;
-    Module['get_first_emval'] = get_first_emval;
   },
 
-  $count_emval_handles__deps: ['$emval_handle_array'],
-  $count_emval_handles: function() {
+  $count_emval_handles__deps: ['$emval_handles'],
+  $count_emval_handles: () => {
     var count = 0;
-    for (var i = 5; i < emval_handle_array.length; ++i) {
-      if (emval_handle_array[i] !== undefined) {
+    for (var i = emval_handles.reserved; i < emval_handles.allocated.length; ++i) {
+      if (emval_handles.allocated[i] !== undefined) {
         ++count;
       }
     }
     return count;
   },
 
-  $get_first_emval__deps: ['$emval_handle_array'],
-  $get_first_emval: function() {
-    for (var i = 5; i < emval_handle_array.length; ++i) {
-      if (emval_handle_array[i] !== undefined) {
-        return emval_handle_array[i];
-      }
-    }
-    return null;
-  },
-
   _emval_register_symbol__deps: ['$emval_symbols', '$readLatin1String'],
-  _emval_register_symbol: function(address) {
+  _emval_register_symbol: (address) => {
     emval_symbols[address] = readLatin1String(address);
   },
 
   $getStringOrSymbol__deps: ['$emval_symbols', '$readLatin1String'],
-  $getStringOrSymbol: function(address) {
+  $getStringOrSymbol: (address) => {
     var symbol = emval_symbols[address];
     if (symbol === undefined) {
       return readLatin1String(address);
@@ -69,13 +61,13 @@ var LibraryEmVal = {
     return symbol;
   },
 
-  $Emval__deps: ['$emval_handle_array', '$emval_free_list', '$throwBindingError', '$init_emval'],
+  $Emval__deps: ['$emval_handles', '$throwBindingError', '$init_emval'],
   $Emval: {
     toValue: (handle) => {
       if (!handle) {
           throwBindingError('Cannot use deleted val. handle = ' + handle);
       }
-      return emval_handle_array[handle].value;
+      return emval_handles.get(handle).value;
     },
 
     toHandle: (value) => {
@@ -85,51 +77,40 @@ var LibraryEmVal = {
         case true: return 3;
         case false: return 4;
         default:{
-          var handle = emval_free_list.length ?
-              emval_free_list.pop() :
-              emval_handle_array.length;
-
-          emval_handle_array[handle] = {refcount: 1, value: value};
-          return handle;
+          return emval_handles.allocate({refcount: 1, value: value});
         }
       }
     }
   },
 
-  _emval_incref__sig: 'vp',
-  _emval_incref__deps: ['$emval_handle_array'],
-  _emval_incref: function(handle) {
+  _emval_incref__deps: ['$emval_handles'],
+  _emval_incref: (handle) => {
     if (handle > 4) {
-      emval_handle_array[handle].refcount += 1;
+      emval_handles.get(handle).refcount += 1;
     }
   },
 
-  _emval_decref__sig: 'vp',
-  _emval_decref__deps: ['$emval_free_list', '$emval_handle_array'],
-  _emval_decref: function(handle) {
-    if (handle > 4 && 0 === --emval_handle_array[handle].refcount) {
-      emval_handle_array[handle] = undefined;
-      emval_free_list.push(handle);
+  _emval_decref__deps: ['$emval_handles'],
+  _emval_decref: (handle) => {
+    if (handle >= emval_handles.reserved && 0 === --emval_handles.get(handle).refcount) {
+      emval_handles.free(handle);
     }
   },
 
-  _emval_run_destructors__sig: 'vp',
   _emval_run_destructors__deps: ['_emval_decref', '$Emval', '$runDestructors'],
-  _emval_run_destructors: function(handle) {
+  _emval_run_destructors: (handle) => {
     var destructors = Emval.toValue(handle);
     runDestructors(destructors);
     __emval_decref(handle);
   },
 
-  _emval_new_array__sig: 'p',
   _emval_new_array__deps: ['$Emval'],
-  _emval_new_array: function() {
+  _emval_new_array: () => {
     return Emval.toHandle([]);
   },
 
-  _emval_new_array_from_memory_view__sig: 'pp',
   _emval_new_array_from_memory_view__deps: ['$Emval'],
-  _emval_new_array_from_memory_view: function(view) {
+  _emval_new_array_from_memory_view: (view) => {
     view = Emval.toValue(view);
     // using for..loop is faster than Array.from
     var a = new Array(view.length);
@@ -137,33 +118,28 @@ var LibraryEmVal = {
     return Emval.toHandle(a);
   },
 
-  _emval_new_object__sig: 'p',
   _emval_new_object__deps: ['$Emval'],
-  _emval_new_object: function() {
+  _emval_new_object: () => {
     return Emval.toHandle({});
   },
 
-  _emval_new_cstring__sig: 'pp',
   _emval_new_cstring__deps: ['$getStringOrSymbol', '$Emval'],
-  _emval_new_cstring: function(v) {
+  _emval_new_cstring: (v) => {
     return Emval.toHandle(getStringOrSymbol(v));
   },
 
-  _emval_new_u8string__sig: 'pp',
   _emval_new_u8string__deps: ['$Emval'],
-  _emval_new_u8string: function(v) {
+  _emval_new_u8string: (v) => {
     return Emval.toHandle(UTF8ToString(v));
   },
 
-  _emval_new_u16string__sig: 'pp',
   _emval_new_u16string__deps: ['$Emval'],
-  _emval_new_u16string: function(v) {
+  _emval_new_u16string: (v) => {
     return Emval.toHandle(UTF16ToString(v));
   },
 
-  _emval_take_value__sig: 'ppp',
   _emval_take_value__deps: ['$Emval', '$requireRegisteredType'],
-  _emval_take_value: function(type, arg) {
+  _emval_take_value: (type, arg) => {
     type = requireRegisteredType(type, '_emval_take_value');
     var v = type['readValueFromPointer'](arg);
     return Emval.toHandle(v);
@@ -171,7 +147,7 @@ var LibraryEmVal = {
 
   $emval_newers: {}, // arity -> function
   $craftEmvalAllocator__deps: ['$Emval', '$requireRegisteredType'],
-  $craftEmvalAllocator: function(argCount) {
+  $craftEmvalAllocator: (argCount) => {
     /*This function returns a new function that looks like this:
     function emval_allocator_3(constructor, argTypes, args) {
         var argType0 = requireRegisteredType(HEAP32[(argTypes >> 2)], "parameter 0");
@@ -188,7 +164,7 @@ var LibraryEmVal = {
     return function(constructor, argTypes, args) {
       argsList[0] = constructor;
       for (var i = 0; i < argCount; ++i) {
-        var argType = requireRegisteredType({{{ makeGetValue('argTypes', 'i * POINTER_SIZE', '*') }}}, 'parameter ' + i);
+        var argType = requireRegisteredType({{{ makeGetValue('argTypes', 'i * ' + POINTER_SIZE, '*') }}}, 'parameter ' + i);
         argsList[i + 1] = argType['readValueFromPointer'](args);
         args += argType['argPackAdvance'];
       }
@@ -229,9 +205,8 @@ var LibraryEmVal = {
 #endif
   },
 
-  _emval_new__sig: 'ppipp',
   _emval_new__deps: ['$craftEmvalAllocator', '$emval_newers', '$Emval'],
-  _emval_new: function(handle, argCount, argTypes, args) {
+  _emval_new: (handle, argCount, argTypes, args) => {
     handle = Emval.toValue(handle);
 
     var newer = emval_newers[argCount];
@@ -244,7 +219,7 @@ var LibraryEmVal = {
   },
 
 #if !DYNAMIC_EXECUTION
-  $emval_get_global: function() {
+  $emval_get_global: () => {
     if (typeof globalThis == 'object') {
       return globalThis;
     }
@@ -271,7 +246,7 @@ var LibraryEmVal = {
   },
 #else
   // appease jshint (technically this code uses eval)
-  $emval_get_global: function() {
+  $emval_get_global: () => {
     if (typeof globalThis == 'object') {
       return globalThis;
     }
@@ -280,9 +255,8 @@ var LibraryEmVal = {
     })()('return this')();
   },
 #endif
-  _emval_get_global__sig: 'pp',
   _emval_get_global__deps: ['$Emval', '$getStringOrSymbol', '$emval_get_global'],
-  _emval_get_global: function(name) {
+  _emval_get_global: (name) => {
     if (name===0) {
       return Emval.toHandle(emval_get_global());
     } else {
@@ -291,33 +265,29 @@ var LibraryEmVal = {
     }
   },
 
-  _emval_get_module_property__sig: 'pp',
   _emval_get_module_property__deps: ['$getStringOrSymbol', '$Emval'],
-  _emval_get_module_property: function(name) {
+  _emval_get_module_property: (name) => {
     name = getStringOrSymbol(name);
     return Emval.toHandle(Module[name]);
   },
 
-  _emval_get_property__sig: 'ppp',
   _emval_get_property__deps: ['$Emval'],
-  _emval_get_property: function(handle, key) {
+  _emval_get_property: (handle, key) => {
     handle = Emval.toValue(handle);
     key = Emval.toValue(key);
     return Emval.toHandle(handle[key]);
   },
 
-  _emval_set_property__sig: 'vppp',
   _emval_set_property__deps: ['$Emval'],
-  _emval_set_property: function(handle, key, value) {
+  _emval_set_property: (handle, key, value) => {
     handle = Emval.toValue(handle);
     key = Emval.toValue(key);
     value = Emval.toValue(value);
     handle[key] = value;
   },
 
-  _emval_as__sig: 'dppp',
   _emval_as__deps: ['$Emval', '$requireRegisteredType'],
-  _emval_as: function(handle, returnType, destructorsRef) {
+  _emval_as: (handle, returnType, destructorsRef) => {
     handle = Emval.toValue(handle);
     returnType = requireRegisteredType(returnType, 'emval::as');
     var destructors = [];
@@ -327,63 +297,55 @@ var LibraryEmVal = {
   },
 
   _emval_as_int64__deps: ['$Emval', '$requireRegisteredType'],
-  _emval_as_int64__sig: 'dppp',
-  _emval_as_int64: function(handle, returnType, destructorsRef) {
+  _emval_as_int64: (handle, returnType) => {
     handle = Emval.toValue(handle);
     returnType = requireRegisteredType(returnType, 'emval::as');
     return returnType['toWireType'](null, handle);
   },
 
   _emval_as_uint64__deps: ['$Emval', '$requireRegisteredType'],
-  _emval_as_uint64__sig: 'dppp',
-  _emval_as_uint64: function(handle, returnType, destructorsRef) {
+  _emval_as_uint64: (handle, returnType) => {
     handle = Emval.toValue(handle);
     returnType = requireRegisteredType(returnType, 'emval::as');
     return returnType['toWireType'](null, handle);
   },
 
   _emval_equals__deps: ['$Emval'],
-  _emval_equals__sig: 'ipp',
-  _emval_equals: function(first, second) {
+  _emval_equals: (first, second) => {
     first = Emval.toValue(first);
     second = Emval.toValue(second);
     return first == second;
   },
 
   _emval_strictly_equals__deps: ['$Emval'],
-  _emval_strictly_equals__sig: 'ipp',
-  _emval_strictly_equals: function(first, second) {
+  _emval_strictly_equals: (first, second) => {
     first = Emval.toValue(first);
     second = Emval.toValue(second);
     return first === second;
   },
 
   _emval_greater_than__deps: ['$Emval'],
-  _emval_greater_than__sig: 'ipp',
-  _emval_greater_than: function(first, second) {
+  _emval_greater_than: (first, second) => {
     first = Emval.toValue(first);
     second = Emval.toValue(second);
     return first > second;
   },
 
   _emval_less_than__deps: ['$Emval'],
-  _emval_less_than__sig: 'ipp',
-  _emval_less_than: function(first, second) {
+  _emval_less_than: (first, second) => {
     first = Emval.toValue(first);
     second = Emval.toValue(second);
     return first < second;
   },
 
   _emval_not__deps: ['$Emval'],
-  _emval_not__sig: 'ip',
-  _emval_not: function(object) {
+  _emval_not: (object) => {
     object = Emval.toValue(object);
     return !object;
   },
 
-  _emval_call__sig: 'ppppp',
   _emval_call__deps: ['$emval_lookupTypes', '$Emval'],
-  _emval_call: function(handle, argCount, argTypes, argv) {
+  _emval_call: (handle, argCount, argTypes, argv) => {
     handle = Emval.toValue(handle);
     var types = emval_lookupTypes(argCount, argTypes);
 
@@ -399,17 +361,17 @@ var LibraryEmVal = {
   },
 
   $emval_lookupTypes__deps: ['$requireRegisteredType'],
-  $emval_lookupTypes: function(argCount, argTypes) {
+  $emval_lookupTypes: (argCount, argTypes) => {
     var a = new Array(argCount);
     for (var i = 0; i < argCount; ++i) {
-      a[i] = requireRegisteredType({{{ makeGetValue('argTypes', 'i * POINTER_SIZE', '*') }}},
+      a[i] = requireRegisteredType({{{ makeGetValue('argTypes', 'i * ' + POINTER_SIZE, '*') }}},
                                    "parameter " + i);
     }
     return a;
   },
 
   $emval_allocateDestructors__deps: ['$Emval'],
-  $emval_allocateDestructors: function(destructorsRef) {
+  $emval_allocateDestructors: (destructorsRef) => {
     var destructors = [];
     {{{ makeSetValue('destructorsRef', '0', 'Emval.toHandle(destructors)', '*') }}};
     return destructors;
@@ -420,16 +382,21 @@ var LibraryEmVal = {
   $emval_methodCallers: [undefined],
 
   $emval_addMethodCaller__deps: ['$emval_methodCallers'],
-  $emval_addMethodCaller: function(caller) {
+  $emval_addMethodCaller: (caller) => {
     var id = emval_methodCallers.length;
     emval_methodCallers.push(caller);
     return id;
   },
 
-  $emval_registeredMethods: [],
-  _emval_get_method_caller__sig: 'ppp',
-  _emval_get_method_caller__deps: ['$emval_addMethodCaller', '$emval_lookupTypes', '$new_', '$makeLegalFunctionName', '$emval_registeredMethods'],
-  _emval_get_method_caller: function(argCount, argTypes) {
+  $emval_registeredMethods: {},
+  _emval_get_method_caller__deps: [
+    '$emval_addMethodCaller', '$emval_lookupTypes',,
+    '$makeLegalFunctionName', '$emval_registeredMethods',
+#if DYNAMIC_EXECUTION
+    '$newFunc',
+#endif
+  ],
+  _emval_get_method_caller: (argCount, argTypes) => {
     var types = emval_lookupTypes(argCount, argTypes);
     var retType = types[0];
     var signatureName = retType.name + "_$" + types.slice(1).map(function (t) { return t.name; }).join("_") + "$";
@@ -493,7 +460,7 @@ var LibraryEmVal = {
         "};\n";
 
     params.push(functionBody);
-    var invokerFunction = new_(Function, params).apply(null, args);
+    var invokerFunction = newFunc(Function, params).apply(null, args);
 #endif
     returnId = emval_addMethodCaller(invokerFunction);
     emval_registeredMethods[signatureName] = returnId;
@@ -501,78 +468,70 @@ var LibraryEmVal = {
   },
 
   _emval_call_method__deps: ['$emval_allocateDestructors', '$getStringOrSymbol', '$emval_methodCallers', '$Emval'],
-  _emval_call_method__sig: 'dppppp',
-  _emval_call_method: function(caller, handle, methodName, destructorsRef, args) {
+  _emval_call_method: (caller, handle, methodName, destructorsRef, args) => {
     caller = emval_methodCallers[caller];
     handle = Emval.toValue(handle);
     methodName = getStringOrSymbol(methodName);
     return caller(handle, methodName, emval_allocateDestructors(destructorsRef), args);
   },
 
-  _emval_call_void_method__sig: 'vpppp',
   _emval_call_void_method__deps: ['$emval_allocateDestructors', '$getStringOrSymbol', '$emval_methodCallers', '$Emval'],
-  _emval_call_void_method: function(caller, handle, methodName, args) {
+  _emval_call_void_method: (caller, handle, methodName, args) => {
     caller = emval_methodCallers[caller];
     handle = Emval.toValue(handle);
     methodName = getStringOrSymbol(methodName);
     caller(handle, methodName, null, args);
   },
 
-  _emval_typeof__sig: 'pp',
   _emval_typeof__deps: ['$Emval'],
-  _emval_typeof: function(handle) {
+  _emval_typeof: (handle) => {
     handle = Emval.toValue(handle);
     return Emval.toHandle(typeof handle);
   },
 
   _emval_instanceof__deps: ['$Emval'],
-  _emval_instanceof__sig: 'ipp',
-  _emval_instanceof: function(object, constructor) {
+  _emval_instanceof: (object, constructor) => {
     object = Emval.toValue(object);
     constructor = Emval.toValue(constructor);
     return object instanceof constructor;
   },
 
   _emval_is_number__deps: ['$Emval'],
-  _emval_is_number__sig: 'ip',
-  _emval_is_number: function(handle) {
+  _emval_is_number: (handle) => {
     handle = Emval.toValue(handle);
     return typeof handle == 'number';
   },
 
   _emval_is_string__deps: ['$Emval'],
-  _emval_is_string__sig: 'ip',
-  _emval_is_string: function(handle) {
+  _emval_is_string: (handle) => {
     handle = Emval.toValue(handle);
     return typeof handle == 'string';
   },
 
   _emval_in__deps: ['$Emval'],
-  _emval_in__sig: 'ipp',
-  _emval_in: function(item, object) {
+  _emval_in: (item, object) => {
     item = Emval.toValue(item);
     object = Emval.toValue(object);
     return item in object;
   },
 
   _emval_delete__deps: ['$Emval'],
-  _emval_delete__sig: 'ipp',
-  _emval_delete: function(object, property) {
+  _emval_delete: (object, property) => {
     object = Emval.toValue(object);
     property = Emval.toValue(property);
     return delete object[property];
   },
 
   _emval_throw__deps: ['$Emval'],
-  _emval_throw__sig: 'vp',
-  _emval_throw: function(object) {
+  _emval_throw: (object) => {
     object = Emval.toValue(object);
     throw object;
   },
 
 #if ASYNCIFY
   _emval_await__deps: ['$Emval', '$Asyncify'],
-  _emval_await: function(promise) {
+  _emval_await__async: true,
+  _emval_await: (promise) => {
     return Asyncify.handleAsync(() => {
       promise = Emval.toValue(promise);
       return promise.then(Emval.toHandle);
@@ -581,4 +540,4 @@ var LibraryEmVal = {
 #endif
 };
 
-mergeInto(LibraryManager.library, LibraryEmVal);
+addToLibrary(LibraryEmVal);
