@@ -11,9 +11,11 @@
 
 global.addedLibraryItems = {};
 
-// Some JS-implemented library functions are proxied to be called on the main browser thread, if the Emscripten runtime is executing in a Web Worker.
-// Each such proxied function is identified via an ordinal number (this is not the same namespace as function pointers in general).
-global.proxiedFunctionTable = ['null'/* Reserve index 0 for an undefined function*/];
+// Some JS-implemented library functions are proxied to be called on the main
+// browser thread, if the Emscripten runtime is executing in a Web Worker.
+// Each such proxied function is identified via an ordinal number (this is not
+// the same namespace as function pointers in general).
+global.proxiedFunctionTable = [];
 
 // Mangles the given C/JS side function name to assembly level function name (adds an underscore)
 function mangleCSymbolName(f) {
@@ -27,13 +29,7 @@ function mangleCSymbolName(f) {
 function splitter(array, filter) {
   const splitOut = array.filter(filter);
   const leftIn = array.filter((x) => !filter(x));
-  return {leftIn: leftIn, splitOut: splitOut};
-}
-
-// Functions that start with '$' should not be exported to the wasm module.
-// They are intended to be exclusive to JS code only.
-function isJsOnlySymbol(symbol) {
-  return symbol[0] == '$';
+  return { leftIn, splitOut };
 }
 
 function escapeJSONKey(x) {
@@ -42,13 +38,26 @@ function escapeJSONKey(x) {
   return "'" + x + "'";
 }
 
+// JSON.stringify will completely omit function objects.  This function is
+// similar but preserves functions.
 function stringifyWithFunctions(obj) {
   if (typeof obj == 'function') return obj.toString();
   if (obj === null || typeof obj != 'object') return JSON.stringify(obj);
   if (Array.isArray(obj)) {
     return '[' + obj.map(stringifyWithFunctions).join(',') + ']';
   }
-  return '{' + Object.keys(obj).map((key) => escapeJSONKey(key) + ':' + stringifyWithFunctions(obj[key])).join(',') + '}';
+  var rtn = '{\n';
+  for (const [key, value] of Object.entries(obj)) {
+    var str = stringifyWithFunctions(value);
+    // Handle JS method syntax where the function property starts with its own
+    // name. e.g.  foo(a) {},
+    if (typeof value === 'function' && str.startsWith(key)) {
+      rtn += str + ',\n'
+    } else {
+      rtn += escapeJSONKey(key) + ':' + str + ',\n';
+    }
+  }
+  return rtn + '}';
 }
 
 function isDefined(symName) {
@@ -102,59 +111,89 @@ function runJSify() {
   }
   if (INCLUDE_FULL_LIBRARY) {
     for (const key of Object.keys(LibraryManager.library)) {
-      if (!isJsLibraryConfigIdentifier(key)) {
+      if (!isDecorator(key)) {
         symbolsNeeded.push(key);
       }
     }
   }
 
-  function convertPointerParams(symbol, snippet, sig) {
-    // Automatically convert any incoming pointer arguments from BigInt
-    // to double (this limits the range to int53).
-    // And convert the return value if the function returns a pointer.
-    return modifyFunction(snippet, (name, args, body) => {
+  function handleI64Signatures(symbol, snippet, sig, i53abi) {
+    // Handle i64 paramaters and return values.
+    //
+    // When WASM_BIGINT is enabled these arrive as BigInt values which we
+    // convert to int53 JS numbers.  If necessary, we also convert the return
+    // value back into a BigInt.
+    //
+    // When WASM_BIGINT is not enabled we receive i64 values as a pair of i32
+    // numbers which is coverted to single int53 number.  In necessary, we also
+    // split the return value into a pair of i32 numbers.
+    return modifyJSFunction(snippet, (args, body, async_, oneliner) => {
       let argLines = args.split('\n');
       argLines = argLines.map((line) => line.split('//')[0]);
       const argNames = argLines.join(' ').split(',').map((name) => name.trim());
-      let newArgs = [];
+      const newArgs = [];
+      let innerArgs = [];
       let argConvertions = '';
       for (let i = 1; i < sig.length; i++) {
         const name = argNames[i - 1];
         if (!name) {
-          error(`convertPointerParams: missing name for argument ${i} in ${symbol}`);
+          error(`handleI64Signatures: missing name for argument ${i} in ${symbol}`);
           return snippet;
         }
-        if (sig[i] == 'p') {
-          argConvertions += `  ${name} = Number(${name});\n`;
-          newArgs.push(`Number(${name})`);
+        if (WASM_BIGINT && ((MEMORY64 && sig[i] == 'p') || (i53abi && sig[i] == 'j'))) {
+          argConvertions += `  ${receiveI64ParamAsI53(name, undefined, false)}\n`;
         } else {
-          newArgs.push(name);
+          if (sig[i] == 'j' && i53abi) {
+            argConvertions += `  ${receiveI64ParamAsI53(name, undefined, false)}\n`;
+            newArgs.push(defineI64Param(name));
+          } else if (sig[i] == 'p' && CAN_ADDRESS_2GB) {
+            argConvertions += `  ${name} >>>= 0;\n`;
+            newArgs.push(name);
+          } else {
+            newArgs.push(name);
+          }
         }
       }
 
-      if (sig[0] == 'p') {
-        // For functions that return a pointer we need to convert
-        // the return value too, which means we need to wrap the
-        // body in an inner function.
-        newArgs = newArgs.join(',');
+      var origArgs = args;
+      if (!WASM_BIGINT) {
+        args = newArgs.join(',');
+      }
+
+      if ((sig[0] == 'j' && i53abi) || (sig[0] == 'p' && MEMORY64)) {
+        // For functions that where we need to mutate the return value, we
+        // also need to wrap the body in an inner function.
+        if (oneliner) {
+          if (argConvertions) {
+            return `${async_}(${args}) => {
+${argConvertions}
+  return ${makeReturn64(body)};
+}`
+          }
+          return `${async_}(${args}) => ${makeReturn64(body)};`
+        }
         return `\
-function ${name}(${args}) {
-  var ret = ((${args}) => { ${body} })(${newArgs});
-  return BigInt(ret);
+${async_}function(${args}) {
+${argConvertions}
+  var ret = (() => { ${body} })();
+  return ${makeReturn64('ret')};
 }`;
       }
 
       // Otherwise no inner function is needed and we covert the arguments
       // before executing the function body.
+      if (oneliner) {
+        body = `return ${body}`;
+      }
       return `\
-function ${name}(${args}) {
+${async_}function(${args}) {
 ${argConvertions}
   ${body};
 }`;
     });
   }
 
-  function processLibraryFunction(snippet, symbol, mangled, deps) {
+  function processLibraryFunction(snippet, symbol, mangled, deps, isStub) {
     // It is possible that when printing the function as a string on Windows,
     // the js interpreter we are in returns the string with Windows line endings
     // \r\n. This is undesirable, since line endings are managed in the form \n
@@ -162,13 +201,14 @@ ${argConvertions}
     // uniform.
     snippet = snippet.toString().replace(/\r\n/gm, '\n');
 
-    // name the function; overwrite if it's already named
-    snippet = snippet.replace(/function(?:\s+([^(]+))?\s*\(/, 'function ' + mangled + '(');
+    if (isStub) {
+      return snippet;
+    }
 
     // apply LIBRARY_DEBUG if relevant
     if (LIBRARY_DEBUG && !isJsOnlySymbol(symbol)) {
-      snippet = modifyFunction(snippet, (name, args, body) => `\
-function ${name}(${args}) {
+      snippet = modifyJSFunction(snippet, (args, body, async) => `\
+function(${args}) {
   var ret = (function() { if (runtimeDebug) err("[library call:${mangled}: " + Array.prototype.slice.call(arguments).map(prettyPrint) + "]");
   ${body}
   }).apply(this, arguments);
@@ -177,26 +217,39 @@ function ${name}(${args}) {
 }`);
     }
 
-    if (SHARED_MEMORY) {
-      const proxyingMode = LibraryManager.library[symbol + '__proxy'];
-      if (proxyingMode) {
-        if (proxyingMode !== 'sync' && proxyingMode !== 'async') {
-          throw new Error(`Invalid proxyingMode ${symbol}__proxy: '${proxyingMode}' specified!`);
-        }
+    const sig = LibraryManager.library[symbol + '__sig'];
+    const i53abi = LibraryManager.library[symbol + '__i53abi'];
+    if (sig &&
+        ((i53abi && sig.includes('j')) || ((MEMORY64 || CAN_ADDRESS_2GB) && sig.includes('p')))) {
+      snippet = handleI64Signatures(symbol, snippet, sig, i53abi);
+      i53ConversionDeps.forEach((d) => deps.push(d))
+    }
+
+    const proxyingMode = LibraryManager.library[symbol + '__proxy'];
+    if (proxyingMode) {
+      if (proxyingMode !== 'sync' && proxyingMode !== 'async' && proxyingMode !== 'none') {
+        throw new Error(`Invalid proxyingMode ${symbol}__proxy: '${proxyingMode}' specified!`);
+      }
+      if (SHARED_MEMORY) {
         const sync = proxyingMode === 'sync';
         if (PTHREADS) {
-          snippet = modifyFunction(snippet, (name, args, body) => `
-function ${name}(${args}) {
+          snippet = modifyJSFunction(snippet, (args, body, async_, oneliner) => {
+            if (oneliner) {
+              body = `return ${body}`;
+            }
+            return `
+function(${args}) {
 if (ENVIRONMENT_IS_PTHREAD)
   return proxyToMainThread(${proxiedFunctionTable.length}, ${+sync}${args ? ', ' : ''}${args});
 ${body}
-}\n`);
+}\n`
+          });
         } else if (WASM_WORKERS && ASSERTIONS) {
           // In ASSERTIONS builds add runtime checks that proxied functions are not attempted to be called in Wasm Workers
           // (since there is no automatic proxying architecture available)
-          snippet = modifyFunction(snippet, (name, args, body) => `
-function ${name}(${args}) {
-  assert(!ENVIRONMENT_IS_WASM_WORKER, "Attempted to call proxied function '${name}' in a Wasm Worker, but in Wasm Worker enabled builds, proxied function architecture is not available!");
+          snippet = modifyJSFunction(snippet, (args, body) => `
+function(${args}) {
+  assert(!ENVIRONMENT_IS_WASM_WORKER, "Attempted to call proxied function '${mangled}' in a Wasm Worker, but in Wasm Worker enabled builds, proxied function architecture is not available!");
   ${body}
 }\n`);
         }
@@ -204,13 +257,6 @@ function ${name}(${args}) {
       }
     }
 
-    if (MEMORY64) {
-      const sig = LibraryManager.library[symbol + '__sig'];
-      if (sig && sig.includes('p')) {
-        snippet = convertPointerParams(symbol, snippet, sig);
-      }
-    }
-
     return snippet;
   }
 
@@ -239,15 +285,18 @@ function ${name}(${args}) {
     // In LLVM, exceptions generate a set of functions of form
     // __cxa_find_matching_catch_1(), __cxa_find_matching_catch_2(), etc.  where
     // the number specifies the number of arguments. In Emscripten, route all
-    // these to a single function '__cxa_find_matching_catch' that variadically
-    // processes all of these functions using JS 'arguments' object.
+    // these to a single function 'findMatchingCatch' that takes an array
+    // of argument.
     if (!WASM_EXCEPTIONS && symbol.startsWith('__cxa_find_matching_catch_')) {
       if (DISABLE_EXCEPTION_THROWING) {
         error('DISABLE_EXCEPTION_THROWING was set (likely due to -fno-exceptions), which means no C++ exception throwing support code is linked in, but exception catching code appears. Either do not set DISABLE_EXCEPTION_THROWING (if you do want exception throwing) or compile all source files with -fno-except (so that no exceptions support code is required); also make sure DISABLE_EXCEPTION_CATCHING is set to the right value - if you want exceptions, it should be off, and vice versa.');
         return;
       }
-      const num = +symbol.split('_').slice(-1)[0];
-      addCxaCatch(num);
+      if (!(symbol in LibraryManager.library)) {
+        // Create a new __cxa_find_matching_catch variant on demand.
+        const num = +symbol.split('_').slice(-1)[0];
+        addCxaCatch(num);
+      }
       // Continue, with the code below emitting the proper JavaScript based on
       // what we just added to the library.
     }
@@ -260,11 +309,22 @@ function ${name}(${args}) {
 
       // don't process any special identifiers. These are looked up when
       // processing the base name of the identifier.
-      if (isJsLibraryConfigIdentifier(symbol)) {
+      if (isDecorator(symbol)) {
         return;
       }
 
-      const deps = LibraryManager.library[symbol + '__deps'] || [];
+      if (!(symbol + '__deps' in LibraryManager.library)) {
+        LibraryManager.library[symbol + '__deps'] = [];
+      }
+
+      deps = LibraryManager.library[symbol + '__deps'];
+      let sig = LibraryManager.library[symbol + '__sig'];
+      if (!WASM_BIGINT && sig && sig[0] == 'j') {
+        // Without WASM_BIGINT functions that return i64 depend on setTempRet0
+        // to return the upper 32-bits of the result.
+        // See makeReturn64 in parseTools.py.
+        deps.push('setTempRet0');
+      }
 
       let isAsyncFunction = false;
       if (ASYNCIFY) {
@@ -279,7 +339,7 @@ function ${name}(${args}) {
       }
 
       if (symbolsOnly) {
-        if (!isJsOnlySymbol(symbol) && LibraryManager.library.hasOwnProperty(symbol)) {
+        if (LibraryManager.library.hasOwnProperty(symbol)) {
           var value = LibraryManager.library[symbol];
           var resolvedSymbol = symbol;
           // Resolve aliases before looking up deps
@@ -336,11 +396,12 @@ function ${name}(${args}) {
         }
         if (!RELOCATABLE) {
           // emit a stub that will fail at runtime
-          LibraryManager.library[symbol] = new Function(`err('missing function: ${symbol}'); abort(-1);`);
+          LibraryManager.library[symbol] = new Function(`abort('missing function: ${symbol}');`);
           // We have already warned/errored about this function, so for the purposes of Closure use, mute all type checks
           // regarding this function, marking ot a variadic function that can take in anything and return anything.
           // (not useful to warn/error multiple times)
           LibraryManager.library[symbol + '__docs'] = '/** @type {function(...*):?} */';
+          isStub = true;
         } else {
           // Create a stub for this symbol which can later be replaced by the
           // dynamic linker.  If this stub is called before the symbol is
@@ -396,7 +457,7 @@ function ${name}(${args}) {
         addImplicitDeps(snippet, deps);
       } else if (typeof snippet == 'function') {
         isFunction = true;
-        snippet = processLibraryFunction(snippet, symbol, mangled, deps);
+        snippet = processLibraryFunction(snippet, symbol, mangled, deps, isStub);
         addImplicitDeps(snippet, deps);
       }
 
@@ -415,27 +476,32 @@ function ${name}(${args}) {
       }
 
       if (VERBOSE) {
-        printErr(`adding ${mangled} and deps ${deps} : ` + (snippet + '').substr(0, 40));
+        printErr(`adding ${symbol} (referenced by ${dependent})`)
       }
       const deps_list = deps.join("','");
       const identDependents = symbol + `__deps: ['${deps_list}']`;
       function addDependency(dep) {
-        return addFromLibrary(dep, `${identDependents}, referenced by ${dependent}`, dep === aliasTarget);
+        return addFromLibrary(dep, `${symbol}, referenced by ${dependent}`, dep === aliasTarget);
       }
       let contentText;
       if (isFunction) {
         // Emit the body of a JS library function.
         if ((USE_ASAN || USE_LSAN || UBSAN_RUNTIME) && LibraryManager.library[symbol + '__noleakcheck']) {
-          contentText = modifyFunction(snippet, (name, args, body) => `
-function ${name}(${args}) {
-  return withBuiltinMalloc(function() {
-    ${body}
-  });
-}\n`);
+          contentText = modifyJSFunction(snippet, (args, body) => `(${args}) => withBuiltinMalloc(() => {${body}})`);
           deps.push('$withBuiltinMalloc');
         } else {
           contentText = snippet; // Regular JS function that will be executed in the context of the calling thread.
         }
+        // Give the function the correct (mangled) name. Overwrite it if it's
+        // already named.  This must happen after the last call to
+        // modifyJSFunction which could have changed or removed the name.
+        if (contentText.match(/^\s*([^}]*)\s*=>/s)) {
+          // Handle arrow functions
+          contentText = `var ${mangled} = ` + contentText + ';';
+        } else {
+          // Handle regular (non-arrow) functions
+          contentText = contentText.replace(/function(?:\s+([^(]+))?\s*\(/, `function ${mangled}(`);
+        }
       } else if (typeof snippet == 'string' && snippet.startsWith(';')) {
         // In JS libraries
         //   foo: ';[code here verbatim]'
@@ -443,6 +509,8 @@ function ${name}(${args}) {
         //   'var foo;[code here verbatim];'
         contentText = 'var ' + mangled + snippet;
         if (snippet[snippet.length - 1] != ';' && snippet[snippet.length - 1] != '}') contentText += ';';
+      } else if (typeof snippet == 'undefined') {
+        contentText = `var ${mangled};`;
       } else {
         // In JS libraries
         //   foo: '=[value]'
@@ -453,11 +521,10 @@ function ${name}(${args}) {
         }
         contentText = `var ${mangled} = ${snippet};`;
       }
-      const sig = LibraryManager.library[symbol + '__sig'];
       // asm module exports are done in emscripten.py, after the asm module is ready. Here
       // we also export library methods as necessary.
       if ((EXPORT_ALL || EXPORTED_FUNCTIONS.has(mangled)) && !isStub) {
-        contentText += `\nModule["${mangled}"] = ${mangled};`;
+        contentText += `\nModule['${mangled}'] = ${mangled};`;
       }
       // Relocatable code needs signatures to create proper wrappers. Stack
       // switching needs signatures so we can create a proper
@@ -465,6 +532,9 @@ function ${name}(${args}) {
       // TODO: For asyncify we could only add the signatures we actually need,
       //       of async imports/exports.
       if (sig && (RELOCATABLE || ASYNCIFY == 2)) {
+        if (!WASM_BIGINT) {
+          sig = sig[0].replace('j', 'i') + sig.slice(1).replace(/j/g, 'ii');
+        }
         contentText += `\n${mangled}.sig = '${sig}';`;
       }
       if (ASYNCIFY && isAsyncFunction) {
@@ -489,7 +559,7 @@ function ${name}(${args}) {
       return depsText + commentText + contentText;
     }
 
-    const JS = addFromLibrary(symbol, 'top-level compiled C/C++ code');
+    const JS = addFromLibrary(symbol, 'root reference (e.g. compiled C/C++ code)');
     libraryItems.push(JS);
   }
 
@@ -538,39 +608,27 @@ function ${name}(${args}) {
     }
 
     if (PTHREADS) {
-      print('\n // proxiedFunctionTable specifies the list of functions that can be called either synchronously or asynchronously from other threads in postMessage()d or internally queued events. This way a pthread in a Worker can synchronously access e.g. the DOM on the main thread.');
-      print('\nvar proxiedFunctionTable = [' + proxiedFunctionTable.join() + '];\n');
-    }
-
-    if ((SUPPORT_BASE64_EMBEDDING || FORCE_FILESYSTEM) && !MINIMAL_RUNTIME) {
-      includeFile('base64Utils.js');
+      print(`
+// proxiedFunctionTable specifies the list of functions that can be called
+// either synchronously or asynchronously from other threads in postMessage()d
+// or internally queued events. This way a pthread in a Worker can synchronously
+// access e.g. the DOM on the main thread.
+var proxiedFunctionTable = [
+  ${proxiedFunctionTable.join(',\n  ')}
+];
+`);
     }
 
     if (abortExecution) {
       throw Error('Aborting compilation due to previous errors');
     }
 
-    // This is the main 'post' pass. Print out the generated code that we have here, together with the
-    // rest of the output that we started to print out earlier (see comment on the
+    // This is the main 'post' pass. Print out the generated code
+    // that we have here, together with the rest of the output
+    // that we started to print out earlier (see comment on the
     // "Final shape that will be created").
     print('// EMSCRIPTEN_END_FUNCS\n');
 
-    if (HEADLESS) {
-      print('if (!ENVIRONMENT_IS_WEB) {');
-      includeFile('headlessCanvas.js');
-      includeFile('headless.js')
-      print('}');
-    }
-    if (PROXY_TO_WORKER) {
-      print('if (ENVIRONMENT_IS_WORKER) {\n');
-      includeFile('webGLWorker.js');
-      includeFile('proxyWorker.js');
-      print('}');
-    }
-    if (DETERMINISTIC) {
-      includeFile('deterministic.js');
-    }
-
     const postFile = MINIMAL_RUNTIME ? 'postamble_minimal.js' : 'postamble.js';
     includeFile(postFile);
 
@@ -579,11 +637,11 @@ function ${name}(${args}) {
     }
 
     print('//FORWARDED_DATA:' + JSON.stringify({
-      librarySymbols: librarySymbols,
-      warnings: warnings,
+      librarySymbols,
+      warnings,
       asyncFuncs,
       ATINITS: ATINITS.join('\n'),
-      ATMAINS: ATMAINS.join('\n'),
+      ATMAINS: STRICT ? '' : ATMAINS.join('\n'),
       ATEXITS: ATEXITS.join('\n'),
     }));
   }
