@@ -34,11 +34,11 @@ var LibraryEGL = {
 
     stringCache: {},
 
-    setErrorCode: function(code) {
+    setErrorCode(code) {
       EGL.errorCode = code;
     },
 
-    chooseConfig: function(display, attribList, config, config_size, numConfigs) {
+    chooseConfig(display, attribList, config, config_size, numConfigs) {
       if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
         EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
         return 0;
@@ -91,7 +91,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLDisplay EGLAPIENTRY eglGetDisplay(EGLNativeDisplayType display_id);
   eglGetDisplay__proxy: 'sync',
-  eglGetDisplay: function(nativeDisplayType) {
+  eglGetDisplay: (nativeDisplayType) => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     // Note: As a 'conformant' implementation of EGL, we would prefer to init here only if the user
     //       calls this function with EGL_DEFAULT_DISPLAY. Other display IDs would be preferred to be unsupported
@@ -109,7 +109,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglInitialize(EGLDisplay dpy, EGLint *major, EGLint *minor);
   eglInitialize__proxy: 'sync',
-  eglInitialize: function(display, majorVersion, minorVersion) {
+  eglInitialize: (display, majorVersion, minorVersion) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -127,7 +127,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglTerminate(EGLDisplay dpy);
   eglTerminate__proxy: 'sync',
-  eglTerminate: function(display) {
+  eglTerminate: (display) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -142,19 +142,19 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglGetConfigs(EGLDisplay dpy, EGLConfig *configs, EGLint config_size, EGLint *num_config);
   eglGetConfigs__proxy: 'sync',
-  eglGetConfigs: function(display, configs, config_size, numConfigs) {
+  eglGetConfigs: (display, configs, config_size, numConfigs) => {
     return EGL.chooseConfig(display, 0, configs, config_size, numConfigs);
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglChooseConfig(EGLDisplay dpy, const EGLint *attrib_list, EGLConfig *configs, EGLint config_size, EGLint *num_config);
   eglChooseConfig__proxy: 'sync',
-  eglChooseConfig: function(display, attrib_list, configs, config_size, numConfigs) {
+  eglChooseConfig: (display, attrib_list, configs, config_size, numConfigs) => {
     return EGL.chooseConfig(display, attrib_list, configs, config_size, numConfigs);
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglGetConfigAttrib(EGLDisplay dpy, EGLConfig config, EGLint attribute, EGLint *value);
   eglGetConfigAttrib__proxy: 'sync',
-  eglGetConfigAttrib: function(display, config, attribute, value) {
+  eglGetConfigAttrib: (display, config, attribute, value) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -271,7 +271,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLSurface EGLAPIENTRY eglCreateWindowSurface(EGLDisplay dpy, EGLConfig config, EGLNativeWindowType win, const EGLint *attrib_list);
   eglCreateWindowSurface__proxy: 'sync',
-  eglCreateWindowSurface: function(display, config, win, attrib_list) {
+  eglCreateWindowSurface: (display, config, win, attrib_list) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -290,7 +290,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglDestroySurface(EGLDisplay display, EGLSurface surface);
   eglDestroySurface__proxy: 'sync',
-  eglDestroySurface: function(display, surface) {
+  eglDestroySurface: (display, surface) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -313,7 +313,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLContext EGLAPIENTRY eglCreateContext(EGLDisplay dpy, EGLConfig config, EGLContext share_context, const EGLint *attrib_list);
   eglCreateContext__proxy: 'sync',
-  eglCreateContext: function(display, config, hmm, contextAttribs) {
+  eglCreateContext: (display, config, hmm, contextAttribs) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -344,7 +344,7 @@ var LibraryEGL = {
       if (glesContextVersion == 3) {
         err('When initializing GLES3/WebGL2 via EGL, one must build with -sMAX_WEBGL_VERSION=2!');
       } else {
-        err('When initializing GLES2/WebGL1 via EGL, one must pass EGL_CONTEXT_CLIENT_VERSION = 2 to GL context attributes! GLES version ' + glesContextVersion + ' is not supported!');
+        err(`When initializing GLES2/WebGL1 via EGL, one must pass EGL_CONTEXT_CLIENT_VERSION = 2 to GL context attributes! GLES version ${glesContextVersion} is not supported!`);
       }
 #endif
       EGL.setErrorCode(0x3005 /* EGL_BAD_CONFIG */);
@@ -377,7 +377,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglDestroyContext(EGLDisplay dpy, EGLContext context);
   eglDestroyContext__proxy: 'sync',
-  eglDestroyContext: function(display, context) {
+  eglDestroyContext: (display, context) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -397,7 +397,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglQuerySurface(EGLDisplay dpy, EGLSurface surface, EGLint attribute, EGLint *value);
   eglQuerySurface__proxy: 'sync',
-  eglQuerySurface: function(display, surface, attribute, value) {
+  eglQuerySurface: (display, surface, attribute, value) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -463,7 +463,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglQueryContext(EGLDisplay dpy, EGLContext ctx, EGLint attribute, EGLint *value);
   eglQueryContext__proxy: 'sync',
-  eglQueryContext: function(display, context, attribute, value) {
+  eglQueryContext: (display, context, attribute, value) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -502,14 +502,14 @@ var LibraryEGL = {
 
   // EGLAPI EGLint EGLAPIENTRY eglGetError(void);
   eglGetError__proxy: 'sync',
-  eglGetError: function() {
+  eglGetError: () => {
     return EGL.errorCode;
   },
 
   // EGLAPI const char * EGLAPIENTRY eglQueryString(EGLDisplay dpy, EGLint name);
   eglQueryString__deps: ['$stringToNewUTF8'],
   eglQueryString__proxy: 'sync',
-  eglQueryString: function(display, name) {
+  eglQueryString: (display, name) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
@@ -533,7 +533,7 @@ var LibraryEGL = {
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglBindAPI(EGLenum api);
   eglBindAPI__proxy: 'sync',
-  eglBindAPI: function(api) {
+  eglBindAPI: (api) => {
     if (api == 0x30A0 /* EGL_OPENGL_ES_API */) {
       EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
       return 1;
@@ -545,21 +545,21 @@ var LibraryEGL = {
 
   // EGLAPI EGLenum EGLAPIENTRY eglQueryAPI(void);
   eglQueryAPI__proxy: 'sync',
-  eglQueryAPI: function() {
+  eglQueryAPI: () => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 0x30A0; // EGL_OPENGL_ES_API
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglWaitClient(void);
   eglWaitClient__proxy: 'sync',
-  eglWaitClient: function() {
+  eglWaitClient: () => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglWaitNative(EGLint engine);
   eglWaitNative__proxy: 'sync',
-  eglWaitNative: function(nativeEngineId) {
+  eglWaitNative: (nativeEngineId) => {
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
   },
@@ -571,13 +571,13 @@ var LibraryEGL = {
   // EGLAPI EGLBoolean EGLAPIENTRY eglSwapInterval(EGLDisplay dpy, EGLint interval);
   eglSwapInterval__deps: ['emscripten_set_main_loop_timing'],
   eglSwapInterval__proxy: 'sync',
-  eglSwapInterval: function(display, interval) {
+  eglSwapInterval: (display, interval) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0;
     }
-    if (interval == 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 0);
-    else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, interval);
+    if (interval == 0) _emscripten_set_main_loop_timing({{{ cDefs.EM_TIMING_SETTIMEOUT }}}, 0);
+    else _emscripten_set_main_loop_timing({{{ cDefs.EM_TIMING_RAF }}}, interval);
 
     EGL.setErrorCode(0x3000 /* EGL_SUCCESS */);
     return 1;
@@ -586,7 +586,7 @@ var LibraryEGL = {
   // EGLAPI EGLBoolean EGLAPIENTRY eglMakeCurrent(EGLDisplay dpy, EGLSurface draw, EGLSurface read, EGLContext ctx);
   eglMakeCurrent__deps: ['$GL'],
   eglMakeCurrent__proxy: 'sync',
-  eglMakeCurrent: function(display, draw, read, context) {
+  eglMakeCurrent: (display, draw, read, context) => {
     if (display != 62000 /* Magic ID for Emscripten 'default display' */) {
       EGL.setErrorCode(0x3008 /* EGL_BAD_DISPLAY */);
       return 0 /* EGL_FALSE */;
@@ -612,13 +612,13 @@ var LibraryEGL = {
 
   // EGLAPI EGLContext EGLAPIENTRY eglGetCurrentContext(void);
   eglGetCurrentContext__proxy: 'sync',
-  eglGetCurrentContext: function() {
+  eglGetCurrentContext: () => {
     return EGL.currentContext;
   },
 
   // EGLAPI EGLSurface EGLAPIENTRY eglGetCurrentSurface(EGLint readdraw);
   eglGetCurrentSurface__proxy: 'sync',
-  eglGetCurrentSurface: function(readdraw) {
+  eglGetCurrentSurface: (readdraw) => {
     if (readdraw == 0x305A /* EGL_READ */) {
       return EGL.currentReadSurface;
     } else if (readdraw == 0x3059 /* EGL_DRAW */) {
@@ -631,13 +631,13 @@ var LibraryEGL = {
 
   // EGLAPI EGLDisplay EGLAPIENTRY eglGetCurrentDisplay(void);
   eglGetCurrentDisplay__proxy: 'sync',
-  eglGetCurrentDisplay: function() {
+  eglGetCurrentDisplay: () => {
     return EGL.currentContext ? 62000 /* Magic ID for Emscripten 'default display' */ : 0;
   },
 
   // EGLAPI EGLBoolean EGLAPIENTRY eglSwapBuffers(EGLDisplay dpy, EGLSurface surface);
   eglSwapBuffers__proxy: 'sync',
-  eglSwapBuffers: function(dpy, surface) {
+  eglSwapBuffers: (dpy, surface) => {
 #if PROXY_TO_WORKER
     if (Browser.doSwapBuffers) Browser.doSwapBuffers();
 #endif
@@ -660,7 +660,7 @@ var LibraryEGL = {
   },
 
   eglReleaseThread__proxy: 'sync',
-  eglReleaseThread: function() {
+  eglReleaseThread: () => {
     // Equivalent to eglMakeCurrent with EGL_NO_CONTEXT and EGL_NO_SURFACE.
     EGL.currentContext = 0;
     EGL.currentReadSurface = 0;
@@ -675,4 +675,4 @@ var LibraryEGL = {
 
 autoAddDeps(LibraryEGL, '$EGL');
 
-mergeInto(LibraryManager.library, LibraryEGL);
+addToLibrary(LibraryEGL);
