@@ -4,20 +4,14 @@
  * SPDX-License-Identifier: MIT
  */
 
-mergeInto(LibraryManager.library, {
-  emscripten_get_exported_function__sig: 'pp',
-  emscripten_get_exported_function__deps: ['$addFunction'],
-  emscripten_get_exported_function: function(name) {
+addToLibrary({
+  emscripten_get_exported_function__deps: ['$addFunction', '$UTF8ToString'],
+  emscripten_get_exported_function: (name) => {
     name = UTF8ToString(name);
     // Wasm backend does not use C name mangling on exports,
     // so adjust for that manually.
     if (name[0] == '_') name = name.substr(1);
-#if MINIMAL_RUNTIME
-    var exportedFunc = asm[name];
-#else
-    // In regular runtime, exports are available on the Module object.
-    var exportedFunc = Module["asm"][name];
-#endif
+    var exportedFunc = wasmExports[name];
     if (exportedFunc) {
       // Record the created function pointer to each function object,
       // so that if the same function pointer is obtained several times,
@@ -26,7 +20,7 @@ mergeInto(LibraryManager.library, {
       return exportedFunc.ptr;
     }
 #if ASSERTIONS
-    err('No exported function found by name "' + exportedFunc + '"');
+    err(`No exported function found by name "{exportedFunc}"`);
 #endif
     // implicit return 0;
   }
