@@ -1,24 +1,19 @@
-{{{ (function() {
+{{{
   // Helper functions for code generation
   global.html5_gpu = {
-    makeImportExport: function(snake_case, CamelCase) {
-      var s = '';
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_import_' + snake_case + '__sig = "ii";';
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_import_' + snake_case + '__deps = ["$WebGPU", "$JsValStore"];';
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_import_' + snake_case + ' = function(handle) { '
-      s += 'return WebGPU.mgr' + CamelCase + '.create(JsValStore.get(handle));'
-      s += '};';
+    makeImportExport: (snake_case, CamelCase) => {
+      return `
+LibraryHTML5WebGPU.emscripten_webgpu_import_${snake_case}__deps = ['$WebGPU', '$JsValStore'];
+LibraryHTML5WebGPU.emscripten_webgpu_import_${snake_case} = (handle) =>
+  WebGPU.mgr${CamelCase}.create(JsValStore.get(handle));
 
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_export_' + snake_case + '__sig = "ii";';
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_export_' + snake_case + '__deps = ["$WebGPU", "$JsValStore"];';
-      s += 'LibraryHTML5WebGPU.emscripten_webgpu_export_' + snake_case + ' = function(handle) { '
-      s += 'return JsValStore.add(WebGPU.mgr' + CamelCase + '.get(handle));'
-      s += '};';
-      return s;
+LibraryHTML5WebGPU.emscripten_webgpu_export_${snake_case}__deps = ['$WebGPU', '$JsValStore'];
+LibraryHTML5WebGPU.emscripten_webgpu_export_${snake_case} = (handle) =>
+  JsValStore.add(WebGPU.mgr${CamelCase}.get(handle));`
     },
   };
-  return null;
-})(); }}}
+  null;
+}}}
 
 
 var LibraryHTML5WebGPU = {
@@ -26,7 +21,7 @@ var LibraryHTML5WebGPU = {
     values: {},
     next_id: 1,
 
-    add: function (js_val) {
+    add(js_val) {
       var id;
       do {
         id = JsValStore.next_id++;
@@ -36,13 +31,13 @@ var LibraryHTML5WebGPU = {
       JsValStore.values[id] = js_val;
       return id;
     },
-    remove: function (id) {
+    remove(id) {
 #if ASSERTIONS
       assert(id in JsValStore.values);
 #endif
       delete JsValStore.values[id];
     },
-    get: function (id) {
+    get(id) {
 #if ASSERTIONS
       assert(id === 0 || id in JsValStore.values);
 #endif
@@ -51,12 +46,10 @@ var LibraryHTML5WebGPU = {
   },
 
   emscripten_webgpu_release_js_handle__deps: ['$JsValStore'],
-  emscripten_webgpu_release_js_handle: function (id) {
-    JsValStore.remove(id);
-  },
+  emscripten_webgpu_release_js_handle: (id) => JsValStore.remove(id),
 
   emscripten_webgpu_get_device__deps: ['$WebGPU'],
-  emscripten_webgpu_get_device: function() {
+  emscripten_webgpu_get_device: () => {
 #if ASSERTIONS
     assert(Module['preinitializedWebGPUDevice']);
 #endif
@@ -97,4 +90,4 @@ var LibraryHTML5WebGPU = {
 {{{ html5_gpu.makeImportExport('render_bundle_encoder', 'RenderBundleEncoder') }}}
 {{{ html5_gpu.makeImportExport('render_bundle', 'RenderBundle') }}}
 
-mergeInto(LibraryManager.library, LibraryHTML5WebGPU);
+addToLibrary(LibraryHTML5WebGPU);
