@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: MIT
  */
 
-mergeInto(LibraryManager.library, {
+addToLibrary({
 #if ASSERTIONS
   $writeI53ToI64__deps: ['$readI53FromI64', '$readI53FromU64'
 #if MINIMAL_RUNTIME
@@ -18,84 +18,90 @@ mergeInto(LibraryManager.library, {
   // Note that unlike the most other function variants in this library, there is no separate
   // function $writeI53ToU64(): the implementation would be identical, and it is up to the
   // C/C++ side code to interpret the resulting number as signed or unsigned as is desirable.
-  $writeI53ToI64: function(ptr, num) {
-    HEAPU32[ptr>>2] = num;
-    HEAPU32[ptr+4>>2] = (num - HEAPU32[ptr>>2])/4294967296;
+  $writeI53ToI64: (ptr, num) => {
+    {{{ makeSetValue('ptr', 0, 'num', 'u32') }}};
+    var lower = {{{ makeGetValue('ptr', 0, 'u32') }}};
+    {{{ makeSetValue('ptr', 4, '(num - lower)/4294967296', 'u32') }}};
 #if ASSERTIONS
     var deserialized = (num >= 0) ? readI53FromU64(ptr) : readI53FromI64(ptr);
-    if (deserialized != num) warnOnce('writeI53ToI64() out of range: serialized JS Number ' + num + ' to Wasm heap as bytes lo=' + ptrToString(HEAPU32[ptr>>2]) + ', hi=' + ptrToString(HEAPU32[ptr+4>>2]) + ', which deserializes back to ' + deserialized + ' instead!');
+    var offset = {{{ getHeapOffset('ptr', 'u32') }}};
+    if (deserialized != num) warnOnce(`writeI53ToI64() out of range: serialized JS Number ${num} to Wasm heap as bytes lo=${ptrToString(HEAPU32[offset])}, hi=${ptrToString(HEAPU32[offset+1])}, which deserializes back to ${deserialized} instead!`);
 #endif
   },
 
   // Same as writeI53ToI64, but if the double precision number does not fit within the
   // 64-bit number, the number is clamped to range [-2^63, 2^63-1].
-  $writeI53ToI64Clamped: function(ptr, num) {
+  $writeI53ToI64Clamped__deps: ['$writeI53ToI64'],
+  $writeI53ToI64Clamped: (ptr, num) => {
     if (num > 0x7FFFFFFFFFFFFFFF) {
-      HEAPU32[ptr>>2] = 0xFFFFFFFF;
-      HEAPU32[ptr+4>>2] = 0x7FFFFFFF;
+      {{{ makeSetValue('ptr', 0, 0xFFFFFFFF, 'u32') }}};
+      {{{ makeSetValue('ptr', 4, 0x7FFFFFFF, 'u32') }}};
     } else if (num < -0x8000000000000000) {
-      HEAPU32[ptr>>2] = 0;
-      HEAPU32[ptr+4>>2] = 0x80000000;
+      {{{ makeSetValue('ptr', 0, 0, 'u32') }}};
+      {{{ makeSetValue('ptr', 4, 0x80000000, 'u32') }}};
     } else {
-      HEAPU32[ptr>>2] = num;
-      HEAPU32[ptr+4>>2] = (num - HEAPU32[ptr>>2])/4294967296;
+      writeI53ToI64(ptr, num);
     }
   },
 
   // Like writeI53ToI64, but throws if the passed number is out of range of int64.
-  $writeI53ToI64Signaling: function(ptr, num) {
+  $writeI53ToI64Signaling__deps: ['$writeI53ToI64'],
+  $writeI53ToI64Signaling: (ptr, num) => {
     if (num > 0x7FFFFFFFFFFFFFFF || num < -0x8000000000000000) {
 #if ASSERTIONS
-      throw 'RangeError in writeI53ToI64Signaling(): input value ' + num + ' is out of range of int64';
+      throw `RangeError in writeI53ToI64Signaling(): input value ${num} is out of range of int64`;
 #else
-      throw 'RangeError:' + num;
+      throw `RangeError: ${num}`;
 #endif
     }
-    HEAPU32[ptr>>2] = num;
-    HEAPU32[ptr+4>>2] = (num - HEAPU32[ptr>>2])/4294967296;
+    writeI53ToI64(ptr, num);
   },
 
   // Uint64 variant of writeI53ToI64Clamped. Writes the Number to a Uint64 variable on
   // the heap, clamping out of range values to range [0, 2^64-1].
-  $writeI53ToU64Clamped: function(ptr, num) {
-    if (num > 0xFFFFFFFFFFFFFFFF) HEAPU32[ptr>>2] = HEAPU32[ptr+4>>2] = 0xFFFFFFFF;
-    else if (num < 0) HEAPU32[ptr>>2] = HEAPU32[ptr+4>>2] = 0;
-    else {
-      HEAPU32[ptr>>2] = num;
-      HEAPU32[ptr+4>>2] = (num - HEAPU32[ptr>>2])/4294967296;
+  $writeI53ToU64Clamped__deps: ['$writeI53ToI64'],
+  $writeI53ToU64Clamped: (ptr, num) => {
+    if (num > 0xFFFFFFFFFFFFFFFF) {
+      {{{ makeSetValue('ptr', 0, 0xFFFFFFFF, 'u32') }}};
+      {{{ makeSetValue('ptr', 4, 0xFFFFFFFF, 'u32') }}};
+    } else if (num < 0) {
+      {{{ makeSetValue('ptr', 0, 0, 'u32') }}};
+      {{{ makeSetValue('ptr', 4, 0, 'u32') }}};
+    } else {
+      writeI53ToI64(ptr, num);
     }
   },
 
   // Like writeI53ToI64, but throws if the passed number is out of range of uint64.
-  $writeI53ToU64Signaling: function(ptr, num) {
+  $writeI53ToU64Signaling__deps: ['$writeI53ToI64'],
+  $writeI53ToU64Signaling: (ptr, num) => {
     if (num < 0 || num > 0xFFFFFFFFFFFFFFFF) {
 #if ASSERTIONS
-      throw 'RangeError in writeI53ToU64Signaling(): input value ' + num + ' is out of range of uint64';
+      throw `RangeError in writeI53ToU64Signaling(): input value ${num} is out of range of uint64`;
 #else
-      throw 'RangeError:'+num;
+      throw `RangeError: ${num}`;
 #endif
     }
-    HEAPU32[ptr>>2] = num;
-    HEAPU32[ptr+4>>2] = (num - HEAPU32[ptr>>2])/4294967296;
+    writeI53ToI64(ptr, num);
   },
 
   // Reads a 64-bit signed integer from the WebAssembly heap and
   // converts it to a JavaScript Number, which can represent 53 integer bits precisely.
   // TODO: Add $readI53FromI64Signaling() variant.
-  $readI53FromI64: function(ptr) {
-    return HEAPU32[ptr>>2] + HEAP32[ptr+4>>2] * 4294967296;
+  $readI53FromI64: (ptr) => {
+    return {{{ makeGetValue('ptr', 0, 'u32') }}} + {{{ makeGetValue('ptr', 4, 'i32') }}} * 4294967296;
   },
 
   // Reads a 64-bit unsigned integer from the WebAssembly heap and
   // converts it to a JavaScript Number, which can represent 53 integer bits precisely.
   // TODO: Add $readI53FromU64Signaling() variant.
-  $readI53FromU64: function(ptr) {
-    return HEAPU32[ptr>>2] + HEAPU32[ptr+4>>2] * 4294967296;
+  $readI53FromU64: (ptr) => {
+    return {{{ makeGetValue('ptr', 0, 'u32') }}} + {{{ makeGetValue('ptr', 4, 'u32') }}} * 4294967296;
   },
 
   // Converts the given signed 32-bit low-high pair to a JavaScript Number that
   // can represent 53 bits of precision.
-  $convertI32PairToI53: function(lo, hi) {
+  $convertI32PairToI53: (lo, hi) => {
 #if ASSERTIONS
     // This function should not be getting called with too large unsigned numbers
     // in high part (if hi >= 0x7FFFFFFFF, one should have been calling
@@ -108,7 +114,7 @@ mergeInto(LibraryManager.library, {
   // Converts the given signed 32-bit low-high pair to a JavaScript Number that can
   // represent 53 bits of precision. Returns a NaN if the number exceeds the safe
   // integer range representable by a Number (x > 9007199254740992 || x < -9007199254740992)
-  $convertI32PairToI53Checked: function(lo, hi) {
+  $convertI32PairToI53Checked: (lo, hi) => {
 #if ASSERTIONS
     assert(lo == (lo >>> 0) || lo == (lo|0)); // lo should either be a i32 or a u32
     assert(hi === (hi|0));                    // hi should be a i32
@@ -119,7 +125,7 @@ mergeInto(LibraryManager.library, {
   // Converts the given unsigned 32-bit low-high pair to a JavaScript Number that can
   // represent 53 bits of precision.
   // TODO: Add $convertU32PairToI53Checked() variant.
-  $convertU32PairToI53: function(lo, hi) {
+  $convertU32PairToI53: (lo, hi) => {
     return (lo >>> 0) + (hi >>> 0) * 4294967296;
   },
 
@@ -131,7 +137,7 @@ mergeInto(LibraryManager.library, {
   // pointer or size_t and is expected to be withing the int53 range.
   // Returns NaN if the incoming bigint is outside the range.
   $bigintToI53Checked__deps: ['$MAX_INT53', '$MIN_INT53'],
-  $bigintToI53Checked: function(num) {
+  $bigintToI53Checked: (num) => {
     return (num < MIN_INT53 || num > MAX_INT53) ? NaN : Number(num);
   },
 #endif
