@@ -12,11 +12,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_ready_state__deps: ['$WS'],
   emscripten_websocket_get_ready_state__proxy: 'sync',
-  emscripten_websocket_get_ready_state: function(socketId, readyState) {
+  emscripten_websocket_get_ready_state: (socketId, readyState) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_ready_state(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_ready_state(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -27,11 +27,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_buffered_amount__deps: ['$WS'],
   emscripten_websocket_get_buffered_amount__proxy: 'sync',
-  emscripten_websocket_get_buffered_amount: function(socketId, bufferedAmount) {
+  emscripten_websocket_get_buffered_amount: (socketId, bufferedAmount) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_buffered_amount(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_buffered_amount(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -42,11 +42,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_extensions__deps: ['$WS', '$stringToUTF8'],
   emscripten_websocket_get_extensions__proxy: 'sync',
-  emscripten_websocket_get_extensions: function(socketId, extensions, extensionsLength) {
+  emscripten_websocket_get_extensions: (socketId, extensions, extensionsLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_extensions(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_extensions(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -57,11 +57,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_extensions_length__deps: ['$WS'],
   emscripten_websocket_get_extensions_length__proxy: 'sync',
-  emscripten_websocket_get_extensions_length: function(socketId, extensionsLength) {
+  emscripten_websocket_get_extensions_length: (socketId, extensionsLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_extensions_length(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_extensions_length(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -72,11 +72,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_protocol__deps: ['$WS', '$stringToUTF8'],
   emscripten_websocket_get_protocol__proxy: 'sync',
-  emscripten_websocket_get_protocol: function(socketId, protocol, protocolLength) {
+  emscripten_websocket_get_protocol: (socketId, protocol, protocolLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_protocol(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_protocol(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -87,11 +87,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_protocol_length__deps: ['$WS'],
   emscripten_websocket_get_protocol_length__proxy: 'sync',
-  emscripten_websocket_get_protocol_length: function(socketId, protocolLength) {
+  emscripten_websocket_get_protocol_length: (socketId, protocolLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_protocol_length(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_protocol_length(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -102,11 +102,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_url__deps: ['$WS', '$stringToUTF8'],
   emscripten_websocket_get_url__proxy: 'sync',
-  emscripten_websocket_get_url: function(socketId, url, urlLength) {
+  emscripten_websocket_get_url: (socketId, url, urlLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_url(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_url(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -117,11 +117,11 @@ var LibraryWebSocket = {
 
   emscripten_websocket_get_url_length__deps: ['$WS'],
   emscripten_websocket_get_url_length__proxy: 'sync',
-  emscripten_websocket_get_url_length: function(socketId, urlLength) {
+  emscripten_websocket_get_url_length: (socketId, urlLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_get_url_length(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_get_url_length(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
@@ -132,7 +132,7 @@ var LibraryWebSocket = {
 
   emscripten_websocket_set_onopen_callback_on_thread__deps: ['$WS'],
   emscripten_websocket_set_onopen_callback_on_thread__proxy: 'sync',
-  emscripten_websocket_set_onopen_callback_on_thread: function(socketId, userData, callbackFunc, thread) {
+  emscripten_websocket_set_onopen_callback_on_thread: (socketId, userData, callbackFunc, thread) => {
 // TODO:
 //    if (thread == {{{ cDefs.EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD }}} ||
 //      (thread == _pthread_self()) return emscripten_websocket_set_onopen_callback_on_calling_thread(socketId, userData, callbackFunc);
@@ -142,17 +142,17 @@ var LibraryWebSocket = {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_set_onopen_callback(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_set_onopen_callback(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_set_onopen_callback(socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+    dbg(`emscripten_websocket_set_onopen_callback(socketId=${socketId},userData=${userData},callbackFunc='+callbackFunc})`);
 #endif
     socket.onopen = function(e) {
 #if WEBSOCKET_DEBUG
-      dbg('websocket event "open": socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+      dbg(`websocket event "open": socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
       HEAPU32[WS.socketEvent>>2] = socketId;
       {{{ makeDynCall('iiii', 'callbackFunc') }}}(0/*TODO*/, WS.socketEvent, userData);
@@ -162,23 +162,23 @@ var LibraryWebSocket = {
 
   emscripten_websocket_set_onerror_callback_on_thread__deps: ['$WS'],
   emscripten_websocket_set_onerror_callback_on_thread__proxy: 'sync',
-  emscripten_websocket_set_onerror_callback_on_thread: function(socketId, userData, callbackFunc, thread) {
+  emscripten_websocket_set_onerror_callback_on_thread: (socketId, userData, callbackFunc, thread) => {
     if (!WS.socketEvent) WS.socketEvent = _malloc(1024); // TODO: sizeof(EmscriptenWebSocketCloseEvent), which is the largest event struct
 
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_set_onerror_callback(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_set_onerror_callback(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_set_onerror_callback(socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+    dbg(`emscripten_websocket_set_onerror_callback(socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
     socket.onerror = function(e) {
 #if WEBSOCKET_DEBUG
-      dbg('websocket event "error": socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+      dbg(`websocket event "error": socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
       HEAPU32[WS.socketEvent>>2] = socketId;
       {{{ makeDynCall('iiii', 'callbackFunc') }}}(0/*TODO*/, WS.socketEvent, userData);
@@ -188,23 +188,23 @@ var LibraryWebSocket = {
 
   emscripten_websocket_set_onclose_callback_on_thread__deps: ['$WS', '$stringToUTF8'],
   emscripten_websocket_set_onclose_callback_on_thread__proxy: 'sync',
-  emscripten_websocket_set_onclose_callback_on_thread: function(socketId, userData, callbackFunc, thread) {
+  emscripten_websocket_set_onclose_callback_on_thread: (socketId, userData, callbackFunc, thread) => {
     if (!WS.socketEvent) WS.socketEvent = _malloc(1024); // TODO: sizeof(EmscriptenWebSocketCloseEvent), which is the largest event struct
 
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_set_onclose_callback(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_set_onclose_callback(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_set_onclose_callback(socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+    dbg(`emscripten_websocket_set_onclose_callback(socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
     socket.onclose = function(e) {
 #if WEBSOCKET_DEBUG
-      dbg('websocket event "close": socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+      dbg(`websocket event "close": socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
       HEAPU32[WS.socketEvent>>2] = socketId;
       HEAPU32[(WS.socketEvent+4)>>2] = e.wasClean;
@@ -217,31 +217,31 @@ var LibraryWebSocket = {
 
   emscripten_websocket_set_onmessage_callback_on_thread__deps: ['$WS', '$stringToNewUTF8', 'malloc', 'free'],
   emscripten_websocket_set_onmessage_callback_on_thread__proxy: 'sync',
-  emscripten_websocket_set_onmessage_callback_on_thread: function(socketId, userData, callbackFunc, thread) {
+  emscripten_websocket_set_onmessage_callback_on_thread: (socketId, userData, callbackFunc, thread) => {
     if (!WS.socketEvent) WS.socketEvent = _malloc(1024); // TODO: sizeof(EmscriptenWebSocketCloseEvent), which is the largest event struct
 
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_set_onmessage_callback(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_set_onmessage_callback(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_set_onmessage_callback(socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+    dbg(`emscripten_websocket_set_onmessage_callback(socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
     socket.onmessage = function(e) {
 #if WEBSOCKET_DEBUG == 2
-      dbg('websocket event "message": socketId='+socketId+',userData='+userData+',callbackFunc='+callbackFunc+')');
+      dbg(`websocket event "message": socketId=${socketId},userData=${userData},callbackFunc=${callbackFunc})`);
 #endif
       HEAPU32[WS.socketEvent>>2] = socketId;
       if (typeof e.data == 'string') {
         var buf = stringToNewUTF8(e.data);
         var len = lengthBytesUTF8(e.data)+1;
 #if WEBSOCKET_DEBUG
-        var s = (e.data.length < 256) ? e.data : (e.data.substr(0, 256) + ' (' + (e.data.length-256) + ' more characters)');
-        dbg('WebSocket onmessage, received data: "' + e.data + '", ' + e.data.length + ' chars, ' + len + ' bytes encoded as UTF-8: "' + s + '"');
+        var s = (e.data.length < 256) ? e.data : (e.data.substr(0, 256) + ` (${e.data.length-256} more characters)`);
+        dbg(`WebSocket onmessage, received data: "${e.data}", ${e.data.length} chars, ${len} bytes encoded as UTF-8: "${s}"`);
 #endif
         HEAPU32[(WS.socketEvent+12)>>2] = 1; // text data
       } else {
@@ -249,12 +249,12 @@ var LibraryWebSocket = {
         var buf = _malloc(len);
         HEAP8.set(new Uint8Array(e.data), buf);
 #if WEBSOCKET_DEBUG
-        var s = 'WebSocket onmessage, received data: ' + len + ' bytes of binary:';
+        var s = `WebSocket onmessage, received data: ${len} bytes of binary:`;
         for (var i = 0; i < Math.min(len, 256); ++i) s += ' ' + HEAPU8[buf+i].toString(16);
         s += ', "';
         for (var i = 0; i < Math.min(len, 256); ++i) s += (HEAPU8[buf+i] >= 32 && HEAPU8[buf+i] <= 127) ? String.fromCharCode(HEAPU8[buf+i]) : '\uFFFD';
         s += '"';
-        if (len > 256) s + ' ... (' + (len - 256) + ' more bytes)';
+        if (len > 256) s + ` ... (${len - 256} more bytes)`;
 
         dbg(s);
 #endif
@@ -270,7 +270,7 @@ var LibraryWebSocket = {
 
   emscripten_websocket_new__deps: ['$WS'],
   emscripten_websocket_new__proxy: 'sync',
-  emscripten_websocket_new: function(createAttributes) {
+  emscripten_websocket_new: (createAttributes) => {
     if (typeof WebSocket == 'undefined') {
 #if WEBSOCKET_DEBUG
       dbg('emscripten_websocket_new(): WebSocket API is not supported by current browser)');
@@ -298,28 +298,28 @@ var LibraryWebSocket = {
     WS.sockets[socketId] = socket;
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_new(url='+url+', protocols=' + (protocols?UTF8ToString(protocols).split(','):'null') + '): created socket ID ' + socketId + ')');
+    dbg(`emscripten_websocket_new(url=${url}, protocols=${protocols ? UTF8ToString(protocols).split(',') : 'null'}): created socket ID ${socketId})`);
 #endif
     return socketId;
   },
 
   emscripten_websocket_send_utf8_text__deps: ['$WS'],
   emscripten_websocket_send_utf8_text__proxy: 'sync',
-  emscripten_websocket_send_utf8_text: function(socketId, textData) {
+  emscripten_websocket_send_utf8_text: (socketId, textData) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_send_utf8_text(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_send_utf8_text(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
     var str = UTF8ToString(textData);
 #if WEBSOCKET_DEBUG == 2
-    dbg('emscripten_websocket_send_utf8_text(socketId='+socketId+',textData='+ str.length + ' chars, "' + str +'")');
+    dbg(`emscripten_websocket_send_utf8_text(socketId=${socketId},textData=${str.length} chars, "${str}")`);
 #else
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_send_utf8_text(socketId='+socketId+',textData='+ str.length + ' chars, "' + ((str.length > 8) ? (str.substring(0,8) + '...') : str) + '")');
+    dbg(`emscripten_websocket_send_utf8_text(socketId=${socketId},textData=${str.length} ' chars, "${(str.length > 8) ? (str.substring(0,8) + '...') : str}")`);
 #endif
 #endif
     socket.send(str);
@@ -328,24 +328,24 @@ var LibraryWebSocket = {
 
   emscripten_websocket_send_binary__deps: ['$WS'],
   emscripten_websocket_send_binary__proxy: 'sync',
-  emscripten_websocket_send_binary: function(socketId, binaryData, dataLength) {
+  emscripten_websocket_send_binary: (socketId, binaryData, dataLength) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_send_binary(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_send_binary(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    var s = 'data: ' + dataLength + ' bytes of binary:';
+    var s = `data: ${dataLength} bytes of binary:`;
     for (var i = 0; i < Math.min(dataLength, 256); ++i) s += ' '+ HEAPU8[binaryData+i].toString(16);
     s += ', "';
     for (var i = 0; i < Math.min(dataLength, 256); ++i) s += (HEAPU8[binaryData+i] >= 32 && HEAPU8[binaryData+i] <= 127) ? String.fromCharCode(HEAPU8[binaryData+i]) : '\uFFFD';
     s += '"';
-    if (dataLength > 256) s + ' ... (' + (dataLength - 256) + ' more bytes)';
+    if (dataLength > 256) s + ` ... (${dataLength - 256} more bytes)`;
 
-    dbg('emscripten_websocket_send_binary(socketId='+socketId+',binaryData='+binaryData+ ',dataLength='+dataLength+'), ' + s);
+    dbg(`emscripten_websocket_send_binary(socketId=${socketId},binaryData=${binaryData},dataLength=${dataLength}), ${s}`);
 #endif
 #if SHARED_MEMORY
     // TODO: This is temporary to cast a shared Uint8Array to a non-shared Uint8Array. This could be removed if WebSocket API is improved
@@ -359,18 +359,18 @@ var LibraryWebSocket = {
 
   emscripten_websocket_close__deps: ['$WS'],
   emscripten_websocket_close__proxy: 'sync',
-  emscripten_websocket_close: function(socketId, code, reason) {
+  emscripten_websocket_close: (socketId, code, reason) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_close(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_close(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
     var reasonStr = reason ? UTF8ToString(reason) : undefined;
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_close(socketId='+socketId+',code='+code+',reason='+reasonStr+')');
+    dbg(`emscripten_websocket_close(socketId=${socketId},code=${code},reason=${reasonStr})`);
 #endif
     // According to WebSocket specification, only close codes that are recognized have integer values
     // 1000-4999, with 3000-3999 and 4000-4999 denoting user-specified close codes:
@@ -385,17 +385,17 @@ var LibraryWebSocket = {
 
   emscripten_websocket_delete__deps: ['$WS'],
   emscripten_websocket_delete__proxy: 'sync',
-  emscripten_websocket_delete: function(socketId) {
+  emscripten_websocket_delete: (socketId) => {
     var socket = WS.sockets[socketId];
     if (!socket) {
 #if WEBSOCKET_DEBUG
-      dbg('emscripten_websocket_delete(): Invalid socket ID ' + socketId + ' specified!');
+      dbg(`emscripten_websocket_delete(): Invalid socket ID ${socketId} specified!`);
 #endif
       return {{{ cDefs.EMSCRIPTEN_RESULT_INVALID_TARGET }}};
     }
 
 #if WEBSOCKET_DEBUG
-    dbg('emscripten_websocket_delete(socketId='+socketId+')');
+    dbg(`emscripten_websocket_delete(socketId=${socketId})`);
 #endif
     socket.onopen = socket.onerror = socket.onclose = socket.onmessage = null;
     delete WS.sockets[socketId];
@@ -403,14 +403,14 @@ var LibraryWebSocket = {
   },
 
   emscripten_websocket_is_supported__proxy: 'sync',
-  emscripten_websocket_is_supported: function() {
+  emscripten_websocket_is_supported: () => {
     return typeof WebSocket != 'undefined';
   },
 
   emscripten_websocket_deinitialize__deps: ['$WS'],
   emscripten_websocket_deinitialize__proxy: 'sync',
   emscripten_websocket_deinitialize__deps: ['emscripten_websocket_delete'],
-  emscripten_websocket_deinitialize: function() {
+  emscripten_websocket_deinitialize: () => {
 #if WEBSOCKET_DEBUG
     dbg('emscripten_websocket_deinitialize()');
 #endif
@@ -425,4 +425,4 @@ var LibraryWebSocket = {
   }
 }
 
-mergeInto(LibraryManager.library, LibraryWebSocket);
+addToLibrary(LibraryWebSocket);
