@@ -26,19 +26,19 @@ global.LibraryManager = {
   loaded: false,
   libraries: [],
 
-  has: function(name) {
+  has(name) {
     return this.libraries.includes(name);
   },
 
-  load: function() {
+  load() {
     assert(!this.loaded);
     this.loaded = true;
 
     // Core system libraries (always linked against)
     let libraries = [
+      'library_int53.js',
       'library.js',
       'library_sigs.js',
-      'library_int53.js',
       'library_ccall.js',
       'library_addfunction.js',
       'library_formatString.js',
@@ -46,11 +46,9 @@ global.LibraryManager = {
       'library_math.js',
       'library_path.js',
       'library_strings.js',
-      'library_syscall.js',
       'library_html5.js',
       'library_stack_trace.js',
       'library_wasi.js',
-      'library_dylink.js',
       'library_makeDynCall.js',
       'library_eventloop.js',
       'library_promise.js',
@@ -77,36 +75,45 @@ global.LibraryManager = {
       libraries.push('library_autodebug.js');
     }
 
-    if (FILESYSTEM) {
-      // Core filesystem libraries (always linked against, unless -sFILESYSTEM=0 is specified)
-      libraries = libraries.concat([
-        'library_fs_shared.js',
-        'library_fs.js',
-        'library_memfs.js',
-        'library_tty.js',
-        'library_pipefs.js', // ok to include it by default since it's only used if the syscall is used
-        'library_sockfs.js', // ok to include it by default since it's only used if the syscall is used
-      ]);
+    if (!WASMFS) {
+      libraries.push('library_syscall.js');
+    }
 
-      if (NODERAWFS) {
-        // NODERAWFS requires NODEFS
-        if (!JS_LIBRARIES.includes('library_nodefs.js')) {
-          libraries.push('library_nodefs.js');
+    if (RELOCATABLE) {
+      libraries.push('library_dylink.js');
+    }
+
+    if (FILESYSTEM) {
+      libraries.push('library_fs_shared.js');
+      if (WASMFS) {
+        libraries = libraries.concat([
+          'library_wasmfs.js',
+          'library_wasmfs_js_file.js',
+          'library_wasmfs_jsimpl.js',
+          'library_wasmfs_fetch.js',
+          'library_wasmfs_node.js',
+          'library_wasmfs_opfs.js',
+        ]);
+      } else {
+        // Core filesystem libraries (always linked against, unless -sFILESYSTEM=0 is specified)
+        libraries = libraries.concat([
+          'library_fs.js',
+          'library_memfs.js',
+          'library_tty.js',
+          'library_pipefs.js', // ok to include it by default since it's only used if the syscall is used
+          'library_sockfs.js', // ok to include it by default since it's only used if the syscall is used
+        ]);
+
+        if (NODERAWFS) {
+          // NODERAWFS requires NODEFS
+          if (!JS_LIBRARIES.includes('library_nodefs.js')) {
+            libraries.push('library_nodefs.js');
+          }
+          libraries.push('library_noderawfs.js');
+          // NODERAWFS overwrites library_path.js
+          libraries.push('library_nodepath.js');
         }
-        libraries.push('library_noderawfs.js');
-        // NODERAWFS overwrites library_path.js
-        libraries.push('library_nodepath.js');
       }
-    } else if (WASMFS) {
-      libraries = libraries.concat([
-        'library_fs_shared.js',
-        'library_wasmfs.js',
-        'library_wasmfs_js_file.js',
-        'library_wasmfs_jsimpl.js',
-        'library_wasmfs_fetch.js',
-        'library_wasmfs_node.js',
-        'library_wasmfs_opfs.js',
-      ]);
     }
 
     // Additional JS libraries (without AUTO_JS_LIBRARIES, link to these explicitly via -lxxx.js)
@@ -211,9 +218,9 @@ global.LibraryManager = {
       if (isUserLibrary) {
         origLibrary = this.library;
         this.library = new Proxy(this.library, {
-          set: (target, prop, value) => {
+          set(target, prop, value) {
             target[prop] = value;
-            if (!isJsLibraryConfigIdentifier(prop)) {
+            if (!isDecorator(prop)) {
               target[prop + '__user'] = true;
             }
             return true;
@@ -296,7 +303,7 @@ function getUnusedLibarySymbols() {
   const missingSyms = new Set();
   for (const [ident, value] of Object.entries(LibraryManager.library)) {
     if (typeof value === 'function' || typeof value === 'number') {
-      if (ident[0] === '$' && !isJsLibraryConfigIdentifier(ident) && !isInternalSymbol(ident)) {
+      if (isJsOnlySymbol(ident) && !isDecorator(ident) && !isInternalSymbol(ident)) {
         const name = ident.substr(1);
         if (!librarySymbolSet.has(name)) {
           missingSyms.add(name);
@@ -348,7 +355,7 @@ function exportRuntime() {
       } else if (legacyRuntimeElements.has(exported)) {
         exported = legacyRuntimeElements.get(exported);
       }
-      return `Module["${name}"] = ${exported};`;
+      return `Module['${name}'] = ${exported};`;
     }
   }
 
@@ -364,17 +371,17 @@ function exportRuntime() {
     'removeRunDependency',
     'FS_createFolder',
     'FS_createPath',
-    'FS_createDataFile',
     'FS_createLazyFile',
     'FS_createLink',
     'FS_createDevice',
-    'FS_unlink',
+    'FS_readFile',
     'out',
     'err',
     'callMain',
     'abort',
     'keepRuntimeAlive',
     'wasmMemory',
+    'wasmExports',
   ];
 
   // These are actually native wasm functions these days but we allow exporting
@@ -443,8 +450,11 @@ function exportRuntime() {
   // '$ which indicates they are JS methods.
   let runtimeElementsSet = new Set(runtimeElements);
   for (const ident of Object.keys(LibraryManager.library)) {
-    if (ident[0] === '$' && !isJsLibraryConfigIdentifier(ident) && !isInternalSymbol(ident)) {
+    if (isJsOnlySymbol(ident) && !isDecorator(ident) && !isInternalSymbol(ident)) {
       const jsname = ident.substr(1);
+      // Note that this assertion may be hit when a function is moved into the
+      // JS library. In that case the function should be removed from the list
+      // of runtime elements above.
       assert(!runtimeElementsSet.has(jsname), 'runtimeElements contains library symbol: ' + ident);
       runtimeElements.push(jsname);
     }
