@@ -11,6 +11,7 @@
 #define _GNU_SOURCE
 #include <assert.h>
 #include <dlfcn.h>
+#include <fcntl.h>
 #include <pthread.h>
 #include <threads.h>
 #include <stdarg.h>
@@ -19,6 +20,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <unistd.h>
 
 #include <emscripten/console.h>
 #include <emscripten/threading.h>
@@ -32,7 +34,7 @@
 //#define DYLINK_DEBUG
 
 #ifdef DYLINK_DEBUG
-#define dbg(fmt, ...) _emscripten_dbgf(fmt, ##__VA_ARGS__)
+#define dbg(fmt, ...) emscripten_dbgf(fmt, ##__VA_ARGS__)
 #else
 #define dbg(fmt, ...)
 #endif
@@ -81,6 +83,8 @@ static thread_local struct dlevent* thread_local_tail = &main_event;
 static pthread_mutex_t write_lock = PTHREAD_MUTEX_INITIALIZER;
 static thread_local bool skip_dlsync = false;
 
+static void dlsync();
+
 static void do_write_lock() {
   // Once we have the lock we want to avoid automatic code sync as that would
   // result in a deadlock.
@@ -155,6 +159,14 @@ static void load_library_done(struct dso* p) {
       p->table_addr,
       p->table_size);
   new_dlevent(p, -1);
+#ifdef _REENTRANT
+  // Block until all other threads have loaded this module.
+  dlsync();
+#endif
+  // TODO: figure out some way to tell when its safe to free p->file_data.  Its
+  // not safe to do here because some threads could have been alseep then when
+  // the "dlsync" occurred and those threads will synchronize when they wake,
+  // which could be an arbitrarily long time in the future.
 }
 
 static struct dso* load_library_start(const char* name, int flags) {
@@ -169,6 +181,29 @@ static struct dso* load_library_start(const char* name, int flags) {
   p->flags = flags;
   strcpy(p->name, name);
 
+  // If the file exists in the filesystem, load it here into linear memory which
+  // makes the data available to JS, and to other threads.  This data gets
+  // free'd later once all threads have loaded the DSO.
+  struct stat statbuf;
+  if (stat(name, &statbuf) == 0 && S_ISREG(statbuf.st_mode)) {
+    int fd = open(name, O_RDONLY);
+    if (fd >= 0) {
+      off_t size = lseek(fd, 0, SEEK_END);
+      if (size != (off_t)-1) {
+        lseek(fd, 0, SEEK_SET);
+        p->file_data = malloc(size);
+        if (p->file_data) {
+          if (read(fd, p->file_data, size) == size) {
+            p->file_data_size = size;
+          } else {
+            free(p->file_data);
+          }
+        }
+      }
+      close(fd);
+    }
+  }
+
   return p;
 }
 
@@ -209,7 +244,7 @@ static void dlsync_next(struct dlevent* dlevent, em_promise_t promise) {
     dbg("calling _dlsym_catchup_js ....");
     void* success = _dlsym_catchup_js(dlevent->dso, dlevent->sym_index);
     if (!success) {
-      _emscripten_errf("_dlsym_catchup_js failed: %s", dlerror());
+      emscripten_errf("_dlsym_catchup_js failed: %s", dlerror());
       sync_one_onerror(dlevent->dso, promise);
       return;
     }
@@ -253,7 +288,7 @@ bool _emscripten_dlsync_self() {
           p->sym_index);
       void* success = _dlsym_catchup_js(p->dso, p->sym_index);
       if (!success) {
-        _emscripten_errf("_dlsym_catchup_js failed: %s", dlerror());
+        emscripten_errf("_dlsym_catchup_js failed: %s", dlerror());
         return false;
       }
     } else {
@@ -271,7 +306,7 @@ bool _emscripten_dlsync_self() {
         // TODO(sbc): Ideally this would never happen and we could/should
         // abort, but on the main thread (where we don't have sync xhr) its
         // often not possible to syncronously load side module.
-        _emscripten_errf("_dlopen_js failed: %s", dlerror());
+        emscripten_errf("_dlopen_js failed: %s", dlerror());
         return false;
       }
     }
@@ -298,11 +333,21 @@ static void do_thread_sync_out(void* arg) {
   *result = _emscripten_dlsync_self();
 }
 
-// Called once _emscripten_proxy_dlsync completes
-static void done_thread_sync(void* arg) {
+// Called when a thread exists prior to being able to completely sync operation.
+// We can just ignore this case and report success.
+static void thread_sync_cancelled(void* arg) {
+  struct promise_result* info = arg;
+  dbg("thread_sync_cancelled: promise=%p result=%i", info->promise, info->result);
+  emscripten_promise_resolve(info->promise, EM_PROMISE_FULFILL, NULL);
+  emscripten_promise_destroy(info->promise);
+  free(info);
+}
+
+// Called once do_thread_sync completes
+static void thread_sync_done(void* arg) {
   struct promise_result* info = arg;
   em_promise_t promise = info->promise;
-  dbg("done_thread_sync: promise=%p result=%i", promise, info->result);
+  dbg("thread_sync_done: promise=%p result=%i", promise, info->result);
   if (info->result) {
     emscripten_promise_resolve(promise, EM_PROMISE_FULFILL, NULL);
   } else {
@@ -354,8 +399,8 @@ int _emscripten_proxy_dlsync_async(pthread_t target_thread, em_promise_t promise
   int rtn = emscripten_proxy_callback(dlopen_proxying_queue,
                                       target_thread,
                                       do_thread_sync,
-                                      done_thread_sync,
-                                      done_thread_sync,
+                                      thread_sync_done,
+                                      thread_sync_cancelled,
                                       info);
   if (!rtn) {
     // If we failed to proxy, then the target thread is no longer alive and no
@@ -424,10 +469,6 @@ static void dlopen_onsuccess(struct dso* dso, void* user_data) {
       dso->mem_addr,
       dso->mem_size);
   load_library_done(dso);
-#ifdef _REENTRANT
-  // Block until all other threads have loaded this module.
-  dlsync();
-#endif
   do_write_unlock();
   data->onsuccess(data->user_data, dso);
   free(data);
@@ -473,6 +514,29 @@ static int path_find(const char *name, const char *s, char *buf, size_t buf_size
   }
 }
 
+// Resolve filename using LD_LIBRARY_PATH
+static const char* resolve_path(char* buf, const char* file, size_t buflen) {
+  if (!strchr(file, '/')) {
+    const char* env_path = getenv("LD_LIBRARY_PATH");
+    if (env_path && path_find(file, env_path, buf, buflen) == 0) {
+      dbg("dlopen: found in LD_LIBRARY_PATH: %s", buf);
+      return buf;
+    }
+  }
+  return file;
+}
+
+// Search for library name to see if it's already loaded
+static struct dso* find_existing(const char* file) {
+  for (struct dlevent* e = head; e; e = e->next) {
+    if (e->sym_index == -1 && !strcmp(e->dso->name, file)) {
+      dbg("dlopen: already opened: %p", e->dso);
+      return e->dso;
+    }
+  }
+  return NULL;
+}
+
 // Internal version of dlopen with typed return value.
 // Without this, the compiler won't tell us if we have the wrong return type.
 static struct dso* _dlopen(const char* file, int flags) {
@@ -492,25 +556,12 @@ static struct dso* _dlopen(const char* file, int flags) {
   dlsync();
 #endif
 
-  struct dso* p;
-
-  /* Resolve filename using LD_LIBRARY_PATH */
   char buf[2*NAME_MAX+2];
-  if (!strchr(file, '/')) {
-    const char* env_path = getenv("LD_LIBRARY_PATH");
-    if (env_path && path_find(file, env_path, buf, sizeof buf) == 0) {
-      dbg("dlopen: found in LD_LIBRARY_PATH: %s", buf);
-      file = buf;
-    }
-  }
+  file = resolve_path(buf, file, sizeof buf);
 
-  /* Search for the name to see if it's already loaded */
-  for (struct dlevent* e = head; e; e = e->next) {
-    if (e->sym_index == -1 && !strcmp(e->dso->name, file)) {
-      dbg("dlopen: already opened: %p", e->dso);
-      p = e->dso;
-      goto end;
-    }
+  struct dso* p = find_existing(file);
+  if (p) {
+    goto end;
   }
 
   p = load_library_start(file, flags);
@@ -526,10 +577,6 @@ static struct dso* _dlopen(const char* file, int flags) {
   }
   dbg("dlopen_js: success: %p", p);
   load_library_done(p);
-#ifdef _REENTRANT
-  // Block until all other threads have loaded this module.
-  dlsync();
-#endif
 end:
   dbg("dlopen(%s): done: %p", file, p);
   do_write_unlock();
@@ -543,8 +590,9 @@ void* dlopen(const char* file, int flags) {
 
 void emscripten_dlopen(const char* filename, int flags, void* user_data,
                        em_dlopen_callback onsuccess, em_arg_callback_func onerror) {
+  dbg("emscripten_dlopen: %s", filename);
   if (!filename) {
-    onsuccess(user_data, head);
+    onsuccess(user_data, head->dso);
     return;
   }
   do_write_lock();
@@ -552,7 +600,14 @@ void emscripten_dlopen(const char* filename, int flags, void* user_data,
   // Make sure we are in sync before performing any write operations.
   dlsync();
 #endif
-  struct dso* p = load_library_start(filename, flags);
+  char buf[2*NAME_MAX+2];
+  filename = resolve_path(buf, filename, sizeof buf);
+  struct dso* p = find_existing(filename);
+  if (p) {
+    onsuccess(user_data, p);
+    return;
+  }
+  p = load_library_start(filename, flags);
   if (!p) {
     do_write_unlock();
     onerror(user_data);
