@@ -13,6 +13,7 @@ from . import diagnostics
 
 # Subset of settings that take a memory size (i.e. 1Gb, 64kb etc)
 MEM_SIZE_SETTINGS = {
+    'GLOBAL_BASE',
     'STACK_SIZE',
     'TOTAL_STACK',
     'INITIAL_MEMORY',
@@ -94,6 +95,11 @@ INTERNAL_SETTINGS = {
 user_settings: Dict[str, str] = {}
 
 
+def default_setting(name, new_default):
+  if name not in user_settings:
+    setattr(settings, name, new_default)
+
+
 class SettingsManager:
   attrs: Dict[str, Any] = {}
   types: Dict[str, Any] = {}
